Package: sessioninfo
Title: R Session Information
Version: 1.2.4
Authors@R: c(
    person("Gábor", "Csárdi", , "csardi.gabor@gmail.com", role = "cre"),
    person("Hadley", "Wickham", role = "aut"),
    person("Winston", "Chang", role = "aut"),
    person("Robert", "Flight", role = "aut"),
    person("Kirill", "Müller", role = "aut"),
    person("Jim", "Hester", role = "aut"),
    person("R Core team", role = "ctb"),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
  )
Maintainer: Gábor Csárdi <csardi.gabor@gmail.com>
Description: Query and print information about the current R session.  It
    is similar to 'utils::sessionInfo()', but includes more information
    about packages, and where they were installed from.
License: GPL-2
URL: https://sessioninfo.r-lib.org,
        https://github.com/r-lib/sessioninfo
BugReports: https://github.com/r-lib/sessioninfo/issues
Depends: R (>= 3.4)
Imports: cli (>= 3.1.0), tools, utils
Suggests: callr, covr, gh, reticulate, rmarkdown, testthat (>= 3.2.0),
        withr
Config/Needs/website: pkgdown, tidyverse/tidytemplate
Config/testthat/edition: 3
Config/testthat/parallel: true
Config/usethis/last-upkeep: 2025-05-07
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-04 13:12:01 UTC; gaborcsardi
Author: Gábor Csárdi [cre],
  Hadley Wickham [aut],
  Winston Chang [aut],
  Robert Flight [aut],
  Kirill Müller [aut],
  Jim Hester [aut],
  R Core team [ctb],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Repository: CRAN
Date/Publication: 2026-06-04 13:50:02 UTC
Built: R 4.6.0; ; 2026-06-04 14:24:20 UTC; unix

Package: sessioninfo
Title: R Session Information
Version: 1.2.4
Authors@R: c(
    person("Gábor", "Csárdi", , "csardi.gabor@gmail.com", role = "cre"),
    person("Hadley", "Wickham", role = "aut"),
    person("Winston", "Chang", role = "aut"),
    person("Robert", "Flight", role = "aut"),
    person("Kirill", "Müller", role = "aut"),
    person("Jim", "Hester", role = "aut"),
    person("R Core team", role = "ctb"),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
  )
Maintainer: Gábor Csárdi <csardi.gabor@gmail.com>
Description: Query and print information about the current R session.  It
    is similar to 'utils::sessionInfo()', but includes more information
    about packages, and where they were installed from.
License: GPL-2
URL: https://sessioninfo.r-lib.org,
        https://github.com/r-lib/sessioninfo
BugReports: https://github.com/r-lib/sessioninfo/issues
Depends: R (>= 3.4)
Imports: cli (>= 3.1.0), tools, utils
Suggests: callr, covr, gh, reticulate, rmarkdown, testthat (>= 3.2.0),
        withr
Config/Needs/website: pkgdown, tidyverse/tidytemplate
Config/testthat/edition: 3
Config/testthat/parallel: true
Config/usethis/last-upkeep: 2025-05-07
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-04 13:12:01 UTC; gaborcsardi
Author: Gábor Csárdi [cre],
  Hadley Wickham [aut],
  Winston Chang [aut],
  Robert Flight [aut],
  Kirill Müller [aut],
  Jim Hester [aut],
  R Core team [ctb],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Repository: CRAN
Date/Publication: 2026-06-04 13:50:02 UTC

# Generated by roxygen2: do not edit by hand

S3method(as.character,external_info)
S3method(as.character,packages_info)
S3method(as.character,platform_info)
S3method(as.character,python_info)
S3method(as.character,session_info)
S3method(c,platform_info)
S3method(format,external_info)
S3method(format,packages_info)
S3method(format,platform_info)
S3method(format,python_info)
S3method(format,session_diff)
S3method(format,session_info)
S3method(print,external_info)
S3method(print,packages_info)
S3method(print,platform_info)
S3method(print,python_info)
S3method(print,session_diff)
S3method(print,session_info)
export(external_info)
export(os_name)
export(package_info)
export(platform_info)
export(python_info)
export(session_diff)
export(session_info)
importFrom(cli,symbol)
importFrom(utils,packageVersion)

# sessioninfo

> R Session Information

<!-- badges: start -->
[![Lifecycle: stable](https://lifecycle.r-lib.org/articles/figures/lifecycle-stable.svg)](https://lifecycle.r-lib.org/articles/stages.html)
[![R-CMD-check](https://github.com/r-lib/sessioninfo/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/r-lib/sessioninfo/actions/workflows/R-CMD-check.yaml)
[![](https://www.r-pkg.org/badges/version/sessioninfo)](https://www.r-pkg.org/pkg/sessioninfo)
[![CRAN RStudio mirror downloads](https://cranlogs.r-pkg.org/badges/sessioninfo)](https://www.r-pkg.org/pkg/sessioninfo)
[![Codecov test coverage](https://codecov.io/gh/r-lib/sessioninfo/branch/main/graph/badge.svg)](https://app.codecov.io/gh/r-lib/sessioninfo?branch=main)
<!-- badges: end -->

Query and print information about the current R session. It is similar to
`utils::sessionInfo()`, but includes more information about packages, and
where they were installed from.

## Differences from `utils::sessionInfo()`

* Additional platform details: time zone, pandoc version, RStudio version,
  etc.
* Information about package sources, e.g. GitHub repo and hash for packages
  installed from GitHub.
* Highlight package installation problems, e.g. if the loaded and on-disk
  versions are different, if the MD5 checksum of the package DLL is wrong,
  etc.
* Highlight packages from unusual sources.
* Information about external software via `external_info()`.
* Information about the Python configuration if the reticulate package is
  loaded and configured.
* Information about package libraries.
* Compare two session info outputs with the `session_diff()` function.
* Option to show loaded (default), attached or installed packages, or
  the recursive dependencies of the specified packages.

## Installation

Install the released version from CRAN

```r
install.packages("sessioninfo")
```

Or the development version from GitHub:

```r
# install.packages("pak")
pak::pak("r-lib/sessioninfo")
```

## Usage

Example output:

```r
sessioninfo::session_info()
```

<img width="1000" src="https://raw.githubusercontent.com/r-lib/sessioninfo/main/man/figures/session-info2.svg" alt="Screenshot of a terminal window demonstrating example output of the sessioninfo function.">

### Copying to the clipboard

You can use the
[`clipr` package](https://cran.r-project.org/package=clipr) to copy
the session info to the clipboard:

```r
clipr::write_clip(session_info())
```

(The current `clipr` version prints a warning, but you can ignore that.)

### Writing to a file

You can use the `to_file` argument of `session_info()`:

```r
session_info(to_file = "session.log")
```

### External software

```r
sessioninfo::session_info(info = "external")
```

```
═ Session info ═══════════════════════════════════════════════════════════════
─ External software ──────────────────────────────────────────────────────────
 setting        value
 cairo          1.14.12
 cairoFT        2.10.0/2.13.1
 pango
 png            1.6.37
 jpeg           9.4
 tiff           LIBTIFF, Version 4.1.0
 tcl            8.6.6
 curl           7.54.0
 zlib           1.2.11
 bzlib          1.0.6, 6-Sept-2010
 xz             5.2.4
 PCRE           10.34 2019-11-21
 ICU            62.1
 TRE            TRE 0.8.0 R_fixes (BSD)
 iconv          GNU libiconv 1.11
 readline       5.2
 BLAS           /Library/Frameworks/R.framework/Versions/4.1/Resources/lib/libRblas.0.dylib
 lapack         /Library/Frameworks/R.framework/Versions/4.1/Resources/lib/libRlapack.dylib
 lapack_version 3.9.0

──────────────────────────────────────────────────────────────────────────────
```

### Python configuration

```r
sessioninfo::session_info(info = "python")
```

```
═ Session info ═══════════════════════════════════════════════════════════════
─ Python configuration ───────────────────────────────────────────────────────
 python:         /Users/gaborcsardi/Library/r-miniconda/envs/r-reticulate/bin/python
 libpython:      /Users/gaborcsardi/Library/r-miniconda/envs/r-reticulate/lib/libpython3.6m.dylib
 pythonhome:     /Users/gaborcsardi/Library/r-miniconda/envs/r-reticulate:/Users/gaborcsardi/Library/r-miniconda/envs/r-reticulate
 version:        3.6.13 | packaged by conda-forge | (default, Sep 23 2021, 07:55:15)  [GCC Clang 11.1.0]
 numpy:          /Users/gaborcsardi/Library/r-miniconda/envs/r-reticulate/lib/python3.6/site-packages/numpy
 numpy_version:  1.19.5

──────────────────────────────────────────────────────────────────────────────
```

### Comparing session information

`session_diff()` can retrieve the session info from an URL or the clipboard
and compare it to the current session information:

```r
sessioninfo::session_diff(new = "https://github.com/r-lib/sessioninfo/issues/6")
```

```diff
--- local
+++ https://github.com/r-lib/sessioninfo/issues/6
  Session info ──────────────────────────────────────────────────────────────────
  setting  value
  version  R version 4.1.1 (2021-08-10)
  os       macOS Mojave 10.14.6
  system   x86_64, darwin17.0
  ui       X11
  language (EN)
  collate  en_US.UTF-8
  ctype    en_US.UTF-8
  tz       Europe/Madrid
  pandoc   2.7.3 @ /usr/local/bin/pandoc

 ─ Packages ─────────────────────────────────────────────────────────────────────
  package     * version     date (UTC) lib source
- asciicast     1.0.0.9000  2021-10-10 [1] local
- cli           3.0.1.9000  2021-10-13 [1] local
+ cachem        1.0.6       2021-08-19 [1] CRAN (R 4.1.0)
+ callr         3.7.0.9000  2021-10-01 [1] Github (r-lib/callr@ea5c3df)
+ cli           3.0.1.9000  2021-10-07 [1] Github (r-lib/cli@e9758aa)
+ clipr         0.7.1       2020-10-08 [1] CRAN (R 4.1.0)
+ commonmark    1.7         2018-12-01 [1] CRAN (R 4.1.0)
  crayon        1.4.1       2021-02-08 [1] CRAN (R 4.1.0)
- curl          4.3.2       2021-06-23 [1] CRAN (R 4.1.0)
  desc          1.4.0.9000  2021-10-04 [1] local
+ devtools      2.4.2       2021-06-07 [1] CRAN (R 4.1.0)
+ digest        0.6.28      2021-09-23 [1] CRAN (R 4.1.0)
  ellipsis      0.3.2       2021-04-29 [1] CRAN (R 4.1.0)
  fansi         0.5.0       2021-05-25 [1] CRAN (R 4.1.0)
+ fastmap       1.1.0       2021-01-25 [1] CRAN (R 4.1.0)
+ fs            1.5.0       2020-07-31 [1] CRAN (R 4.1.0)
  glue          1.4.2       2021-10-04 [1] local
- jsonlite      1.7.2       2020-12-09 [1] CRAN (R 4.1.0)
+ knitr         1.34        2021-09-09 [1] CRAN (R 4.1.0)
  lifecycle     1.0.1       2021-09-24 [1] CRAN (R 4.1.0)
  magrittr      2.0.1       2020-11-17 [1] CRAN (R 4.1.0)
+ memoise       2.0.0       2021-01-26 [1] CRAN (R 4.1.0)
  pillar        1.6.3       2021-09-26 [1] CRAN (R 4.1.1)
+ pkgbuild      1.2.0       2020-12-15 [1] CRAN (R 4.1.0)
  pkgconfig     2.0.3       2019-09-22 [1] CRAN (R 4.1.0)
  pkgload       1.2.2       2021-09-11 [1] CRAN (R 4.1.0)
  prettycode    1.1.0       2019-12-16 [1] CRAN (R 4.1.0)
+ prettyunits   1.1.1       2020-01-24 [1] CRAN (R 4.1.0)
  processx      3.5.2.9000  2021-09-15 [1] local
  prompt        1.0.0       2021-03-02 [1] local
  ps            1.6.0       2021-02-28 [1] CRAN (R 4.1.0)
+ purrr         0.3.4       2020-04-17 [1] CRAN (R 4.1.0)
  R6            2.5.1       2021-08-19 [1] CRAN (R 4.1.0)
- Rcpp          1.0.7       2021-07-07 [1] CRAN (R 4.1.0)
- rlang         0.4.11      2021-04-30 [1] CRAN (R 4.1.0)
+ remotes       2.4.0       2021-06-02 [1] CRAN (R 4.1.0)
+ rlang         0.99.0.9000 2021-10-07 [1] Github (r-lib/rlang@3ba19df)
+ roxygen2      7.1.2       2021-10-04 [1] local
  rprojroot     2.0.2       2020-11-15 [1] CRAN (R 4.1.0)
- sessioninfo * 1.1.1.9000  2021-10-13 [?] load_all()
- testthat    * 3.1.0       2021-10-04 [1] CRAN (R 4.1.0)
- tibble        3.1.5       2021-09-30 [1] CRAN (R 4.1.0)
+ rstudioapi    0.13        2020-11-12 [1] CRAN (R 4.1.0)
+ sessioninfo * 1.1.1.9000  2021-10-05 [?] load_all()
+ stringi       1.7.4       2021-08-25 [1] CRAN (R 4.1.0)
+ stringr       1.4.0       2019-02-10 [1] CRAN (R 4.1.0)
+ testthat    * 3.0.4       2021-07-01 [1] CRAN (R 4.1.0)
+ tibble        3.1.4       2021-08-25 [1] CRAN (R 4.1.0)
+ usethis       2.0.1       2021-02-10 [1] CRAN (R 4.1.0)
  utf8          1.2.2       2021-07-24 [1] CRAN (R 4.1.0)
- uuid          0.1-4       2020-02-26 [1] CRAN (R 4.1.0)
- V8            3.4.2       2021-05-01 [1] CRAN (R 4.1.0)
  vctrs         0.3.8       2021-04-29 [1] CRAN (R 4.1.0)
  withr         2.4.2       2021-04-18 [1] CRAN (R 4.1.0)
+ xfun          0.26        2021-09-14 [1] CRAN (R 4.1.0)
+ xml2          1.3.2       2020-04-23 [1] CRAN (R 4.1.0)

  [1] /Users/gaborcsardi/Library/R/x86_64/4.1/library
  [2] /Library/Frameworks/R.framework/Versions/4.1/Resources/library
```

## Code of Conduct

Please note that the sessioninfo project is released with a
[Contributor Code of Conduct](
  https://sessioninfo.r-lib.org/dev/CODE_OF_CONDUCT.html). By contributing to
this project, you agree to abide by its terms.

## License

GPL-2

# sessioninfo

> R Session Information

<!-- badges: start -->
[![Lifecycle: stable](https://lifecycle.r-lib.org/articles/figures/lifecycle-stable.svg)](https://lifecycle.r-lib.org/articles/stages.html)
[![R-CMD-check](https://github.com/r-lib/sessioninfo/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/r-lib/sessioninfo/actions/workflows/R-CMD-check.yaml)
[![](https://www.r-pkg.org/badges/version/sessioninfo)](https://www.r-pkg.org/pkg/sessioninfo)
[![CRAN RStudio mirror downloads](https://cranlogs.r-pkg.org/badges/sessioninfo)](https://www.r-pkg.org/pkg/sessioninfo)
[![Codecov test coverage](https://codecov.io/gh/r-lib/sessioninfo/graph/badge.svg)](https://app.codecov.io/gh/r-lib/sessioninfo)
<!-- badges: end -->

Query and print information about the current R session. It is similar to
`utils::sessionInfo()`, but includes more information about packages, and
where they were installed from.

## Differences from `utils::sessionInfo()`

* Additional platform details: time zone, pandoc version, RStudio version,
  etc.
* Information about package sources, e.g. GitHub repo and hash for packages
  installed from GitHub.
* Highlight package installation problems, e.g. if the loaded and on-disk
  versions are different, if the MD5 checksum of the package DLL is wrong,
  etc.
* Highlight packages from unusual sources.
* Information about external software via `external_info()`.
* Information about the Python configuration if the reticulate package is
  loaded and configured.
* Information about package libraries.
* Compare two session info outputs with the `session_diff()` function.
* Option to show loaded (default), attached or installed packages, or
  the recursive dependencies of the specified packages.

## Installation

Install the released version from CRAN

```r
install.packages("sessioninfo")
```

Or the development version from GitHub:

```r
# install.packages("pak")
pak::pak("r-lib/sessioninfo")
```

## Usage

Example output:

```r
sessioninfo::session_info()
```

<img width="1000" src="https://raw.githubusercontent.com/r-lib/sessioninfo/main/man/figures/session-info2.svg" alt="Screenshot of a terminal window demonstrating example output of the sessioninfo function.">

### Copying to the clipboard

You can use the
[`clipr` package](https://cran.r-project.org/package=clipr) to copy
the session info to the clipboard:

```r
clipr::write_clip(session_info())
```

(The current `clipr` version prints a warning, but you can ignore that.)

### Writing to a file

You can use the `to_file` argument of `session_info()`:

```r
session_info(to_file = "session.log")
```

### External software

```r
sessioninfo::session_info(info = "external")
```

```
═ Session info ═══════════════════════════════════════════════════════════════
─ External software ──────────────────────────────────────────────────────────
 setting        value
 cairo          1.14.12
 cairoFT        2.10.0/2.13.1
 pango
 png            1.6.37
 jpeg           9.4
 tiff           LIBTIFF, Version 4.1.0
 tcl            8.6.6
 curl           7.54.0
 zlib           1.2.11
 bzlib          1.0.6, 6-Sept-2010
 xz             5.2.4
 PCRE           10.34 2019-11-21
 ICU            62.1
 TRE            TRE 0.8.0 R_fixes (BSD)
 iconv          GNU libiconv 1.11
 readline       5.2
 BLAS           /Library/Frameworks/R.framework/Versions/4.1/Resources/lib/libRblas.0.dylib
 lapack         /Library/Frameworks/R.framework/Versions/4.1/Resources/lib/libRlapack.dylib
 lapack_version 3.9.0

──────────────────────────────────────────────────────────────────────────────
```

### Python configuration

```r
sessioninfo::session_info(info = "python")
```

```
═ Session info ═══════════════════════════════════════════════════════════════
─ Python configuration ───────────────────────────────────────────────────────
 python:         /Users/gaborcsardi/Library/r-miniconda/envs/r-reticulate/bin/python
 libpython:      /Users/gaborcsardi/Library/r-miniconda/envs/r-reticulate/lib/libpython3.6m.dylib
 pythonhome:     /Users/gaborcsardi/Library/r-miniconda/envs/r-reticulate:/Users/gaborcsardi/Library/r-miniconda/envs/r-reticulate
 version:        3.6.13 | packaged by conda-forge | (default, Sep 23 2021, 07:55:15)  [GCC Clang 11.1.0]
 numpy:          /Users/gaborcsardi/Library/r-miniconda/envs/r-reticulate/lib/python3.6/site-packages/numpy
 numpy_version:  1.19.5

──────────────────────────────────────────────────────────────────────────────
```

### Comparing session information

`session_diff()` can retrieve the session info from an URL or the clipboard
and compare it to the current session information:

```r
sessioninfo::session_diff(new = "https://github.com/r-lib/sessioninfo/issues/6")
```

```diff
--- local
+++ https://github.com/r-lib/sessioninfo/issues/6
  Session info ──────────────────────────────────────────────────────────────────
  setting  value
  version  R version 4.1.1 (2021-08-10)
  os       macOS Mojave 10.14.6
  system   x86_64, darwin17.0
  ui       X11
  language (EN)
  collate  en_US.UTF-8
  ctype    en_US.UTF-8
  tz       Europe/Madrid
  pandoc   2.7.3 @ /usr/local/bin/pandoc

 ─ Packages ─────────────────────────────────────────────────────────────────────
  package     * version     date (UTC) lib source
- asciicast     1.0.0.9000  2021-10-10 [1] local
- cli           3.0.1.9000  2021-10-13 [1] local
+ cachem        1.0.6       2021-08-19 [1] CRAN (R 4.1.0)
+ callr         3.7.0.9000  2021-10-01 [1] Github (r-lib/callr@ea5c3df)
+ cli           3.0.1.9000  2021-10-07 [1] Github (r-lib/cli@e9758aa)
+ clipr         0.7.1       2020-10-08 [1] CRAN (R 4.1.0)
+ commonmark    1.7         2018-12-01 [1] CRAN (R 4.1.0)
  crayon        1.4.1       2021-02-08 [1] CRAN (R 4.1.0)
- curl          4.3.2       2021-06-23 [1] CRAN (R 4.1.0)
  desc          1.4.0.9000  2021-10-04 [1] local
+ devtools      2.4.2       2021-06-07 [1] CRAN (R 4.1.0)
+ digest        0.6.28      2021-09-23 [1] CRAN (R 4.1.0)
  ellipsis      0.3.2       2021-04-29 [1] CRAN (R 4.1.0)
  fansi         0.5.0       2021-05-25 [1] CRAN (R 4.1.0)
+ fastmap       1.1.0       2021-01-25 [1] CRAN (R 4.1.0)
+ fs            1.5.0       2020-07-31 [1] CRAN (R 4.1.0)
  glue          1.4.2       2021-10-04 [1] local
- jsonlite      1.7.2       2020-12-09 [1] CRAN (R 4.1.0)
+ knitr         1.34        2021-09-09 [1] CRAN (R 4.1.0)
  lifecycle     1.0.1       2021-09-24 [1] CRAN (R 4.1.0)
  magrittr      2.0.1       2020-11-17 [1] CRAN (R 4.1.0)
+ memoise       2.0.0       2021-01-26 [1] CRAN (R 4.1.0)
  pillar        1.6.3       2021-09-26 [1] CRAN (R 4.1.1)
+ pkgbuild      1.2.0       2020-12-15 [1] CRAN (R 4.1.0)
  pkgconfig     2.0.3       2019-09-22 [1] CRAN (R 4.1.0)
  pkgload       1.2.2       2021-09-11 [1] CRAN (R 4.1.0)
  prettycode    1.1.0       2019-12-16 [1] CRAN (R 4.1.0)
+ prettyunits   1.1.1       2020-01-24 [1] CRAN (R 4.1.0)
  processx      3.5.2.9000  2021-09-15 [1] local
  prompt        1.0.0       2021-03-02 [1] local
  ps            1.6.0       2021-02-28 [1] CRAN (R 4.1.0)
+ purrr         0.3.4       2020-04-17 [1] CRAN (R 4.1.0)
  R6            2.5.1       2021-08-19 [1] CRAN (R 4.1.0)
- Rcpp          1.0.7       2021-07-07 [1] CRAN (R 4.1.0)
- rlang         0.4.11      2021-04-30 [1] CRAN (R 4.1.0)
+ remotes       2.4.0       2021-06-02 [1] CRAN (R 4.1.0)
+ rlang         0.99.0.9000 2021-10-07 [1] Github (r-lib/rlang@3ba19df)
+ roxygen2      7.1.2       2021-10-04 [1] local
  rprojroot     2.0.2       2020-11-15 [1] CRAN (R 4.1.0)
- sessioninfo * 1.1.1.9000  2021-10-13 [?] load_all()
- testthat    * 3.1.0       2021-10-04 [1] CRAN (R 4.1.0)
- tibble        3.1.5       2021-09-30 [1] CRAN (R 4.1.0)
+ rstudioapi    0.13        2020-11-12 [1] CRAN (R 4.1.0)
+ sessioninfo * 1.1.1.9000  2021-10-05 [?] load_all()
+ stringi       1.7.4       2021-08-25 [1] CRAN (R 4.1.0)
+ stringr       1.4.0       2019-02-10 [1] CRAN (R 4.1.0)
+ testthat    * 3.0.4       2021-07-01 [1] CRAN (R 4.1.0)
+ tibble        3.1.4       2021-08-25 [1] CRAN (R 4.1.0)
+ usethis       2.0.1       2021-02-10 [1] CRAN (R 4.1.0)
  utf8          1.2.2       2021-07-24 [1] CRAN (R 4.1.0)
- uuid          0.1-4       2020-02-26 [1] CRAN (R 4.1.0)
- V8            3.4.2       2021-05-01 [1] CRAN (R 4.1.0)
  vctrs         0.3.8       2021-04-29 [1] CRAN (R 4.1.0)
  withr         2.4.2       2021-04-18 [1] CRAN (R 4.1.0)
+ xfun          0.26        2021-09-14 [1] CRAN (R 4.1.0)
+ xml2          1.3.2       2020-04-23 [1] CRAN (R 4.1.0)

  [1] /Users/gaborcsardi/Library/R/x86_64/4.1/library
  [2] /Library/Frameworks/R.framework/Versions/4.1/Resources/library
```

## Code of Conduct

Please note that the sessioninfo project is released with a
[Contributor Code of Conduct](
  https://sessioninfo.r-lib.org/dev/CODE_OF_CONDUCT.html). By contributing to
this project, you agree to abide by its terms.

## License

GPL-2

# sessioninfo

> R Session Information

[![Linux Build Status](https://travis-ci.org/r-lib/sessioninfo.svg?branch=master)](https://travis-ci.org/r-lib/sessioninfo)
[![Windows Build status](https://ci.appveyor.com/api/projects/status/github/r-lib/sessioninfo?svg=true)](https://ci.appveyor.com/project/gaborcsardi/sessioninfo)
[![](http://www.r-pkg.org/badges/version/sessioninfo)](http://www.r-pkg.org/pkg/sessioninfo)
[![CRAN RStudio mirror downloads](http://cranlogs.r-pkg.org/badges/sessioninfo)](http://www.r-pkg.org/pkg/sessioninfo)
[![Coverage Status](https://img.shields.io/codecov/c/github/r-lib/sessioninfo/master.svg)](https://codecov.io/github/r-lib/sessioninfo?branch=master)

Query and print information about the current R session. It is similar to
`utils::sessionInfo()`, but includes more information about packages, and
where they were installed from.

## Installation

```r
devtools::install_github("r-lib/sessioninfo")
```

## Usage

Example output:

```r
sessioninfo::session_info()
```

```r
─ Session info ───────────────────────────────────────────────────────────────
 setting  value
 version  R version 3.5.0 (2018-04-23)
 os       macOS Sierra 10.12.6
 system   x86_64, darwin15.6.0
 ui       X11
 language (EN)
 collate  en_US.UTF-8
 ctype    en_US.UTF-8
 tz       Europe/Brussels
 date     2018-08-09

─ Packages ───────────────────────────────────────────────────────────────────
 package     * version    date       source
 clisymbols    1.2.0      2017-05-21 CRAN (R 3.5.0)
 crayon        1.3.4      2017-09-16 CRAN (R 3.5.0)
 gitty         1.0.0      2018-07-16 Github (gaborcsardi/gitty@67a6e3e)
 memuse        4.0-0      2017-11-10 CRAN (R 3.5.0)
 prettycode  * 1.0.1      2017-12-12 CRAN (R 3.5.0)
 prompt        1.0.0      2018-07-16 Github (gaborcsardi/prompt@e5248b5)
 sessioninfo   1.0.1.9000 2018-08-09 local
 tracer      * 1.0.0      2017-01-28 CRAN (R 3.5.0)
 whisker       0.3-2      2013-04-28 CRAN (R 3.5.0)
 withr         2.1.2      2018-03-15 CRAN (R 3.5.0)
```

### Copying to the clipboard

You can use the
[`clipr` package](https://cran.rstudio.com/web/packages/clipr/) to copy
the session info to the clipboard:

```r
clipr::write_clip(session_info())
```

(The current `clipr` version prints a warning, but you can ignore that.)

## License

GPL-2

#' Print session information
#'
#' This is [utils::sessionInfo()] re-written from scratch to both exclude
#' data that's rarely useful (e.g., the full collate string or base packages
#' loaded) and include stuff you'd like to know (e.g., where a package was
#' installed from).
#'
#' @details
#' Columns in the *printed* package list:
#' * `package`: package name
#' * `*`: whether the package is attached to the search path
#' * `version`: package version. If the version is marked with `(!)` that
#'   means that the loaded and the on-disk version of the package are
#'   different.
#' * `date`: when the package was built, if this information is available.
#'   This is the `Date/Publication` or the `Built` field from
#'   `DESCRIPTION`. (These are usually added automatically by R.)
#'   Sometimes this data is not available, then it is `NA`.
#' * `source`: where the package was built or installed from, if available.
#'   Examples: `CRAN (R 3.3.2)`, `Github (r-lib/pkgbuild@8aab60b)`,
#'   `Bioconductor`, `local`.
#'
#' See [package_info()] for the list of columns in the data frame that
#' is *returned* (as opposed to *printed*).
#'
#' @inheritParams package_info
#' @param info What information to show, it can be `"auto"` to choose
#'   automatically, `"all"` to show everything, or a character vector
#'   with elements from:
#'   * `"platform"`: show platform information via [platform_info()],
#'   * `"packages"`: show package information via [package_info()],
#'   * `"python"`: show Python configuration via [python_info()],
#'   * `"external"`: show information about external software, via
#'      [external_info()].
#' @param to_file Whether to print the session information to a file.
#'   If `TRUE` the name of the file will be `session-info.txt`, but
#'   `to_file` may also be a string to specify the file name.
#'
#' @return
#' A `session_info` object.
#'
#' If `to_file` is not `FALSE` then it is
#' returned invisibly. (To print it to both a file and to the screen,
#' use `(session_info(to_file = TRUE))`.)
#'
#' @export
#' @examplesIf FALSE
#' session_info()
#' session_info("sessioninfo")

session_info <- function(
  pkgs = c("loaded", "attached", "installed")[1],
  include_base = FALSE,
  info = c("auto", "all", "platform", "packages", "python", "external"),
  dependencies = NA,
  to_file = FALSE
) {
  if (missing(info)) info <- "auto"
  choices <- c("platform", "packages", "python", "external")
  if ("all" %in% info) {
    info <- choices
  } else if ("auto" %in% info) {
    info <- c(
      "platform",
      "packages",
      if (should_show_python(pkgs)) "python"
    )
  } else {
    info <- match.arg(info, choices, several.ok = TRUE)
  }

  stopifnot(is_flag(to_file) || is_string(to_file))
  if (is_flag(to_file) && to_file) to_file <- "session-info.txt"

  si <- structure(
    drop_null(list(
      platform = if ("platform" %in% info) platform_info(),
      packages = if ("packages" %in% info) {
        package_info(
          pkgs,
          include_base = include_base,
          dependencies = dependencies
        )
      },
      external = if ("external" %in% info) external_info(),
      python = if ("python" %in% info) python_info()
    )),
    class = c("session_info", "list")
  )

  if (is_string(to_file)) {
    old <- options(cli.num_colors = 1)
    on.exit(options(old), add = TRUE)
    writeLines(format(si), to_file)
    invisible(si)
  } else {
    si
  }
}

#' @export

format.session_info <- function(x, ...) {
  has_platform <- !is.null(x$platform)

  c(
    if (!"platform" %in% names(x)) {
      rule("Session info", double = TRUE)
    },
    if ("platform" %in% names(x)) {
      c(rule(paste("Session info")), format(x$platform), "")
    },
    if ("packages" %in% names(x) && nrow(x$packages) > 0) {
      c(rule("Packages"), format(x$packages), "")
    },
    if ("external" %in% names(x)) {
      c(rule("External software"), format(x$external), "")
    },
    if ("python" %in% names(x)) {
      c(rule("Python configuration"), format(x$python), "")
    },
    rule()
  )
}

#' @export

as.character.session_info <- function(x, ...) {
  old <- options(cli.num_colors = 1)
  on.exit(options(old), add = TRUE)
  format(x, ...)
}

has_emoji <- function() {
  if (isTRUE(opt <- getOption("sessioninfo.emoji"))) {
    TRUE
  } else if (identical(opt, FALSE)) {
    FALSE
  } else if (!cli::is_utf8_output()) {
    FALSE
  } else {
    Sys.info()[["sysname"]] == "Darwin"
  }
}

#' @export

print.session_info <- function(x, ...) {
  cat(format(x), sep = "\n")
}

test_that("session_info", {
  info <- readRDS(paste0("fixtures/devtools-info-", .Platform$OS.type, ".rda"))
  local_mocked_bindings(package_info = function(...) pi)

  si <- session_info()
  expect_equal(si$platform, platform_info())
  expect_equal(si$packages, pi)
})

test_that("print.session_info", {
  si <- session_info()
  expect_output(print(si), "setting[ ]+value")
  expect_output(print(si), "package[ ]+\\* version[ ]+date[ ][(]UTC[)][ ]+lib[ ]+source")
})

test_that("`info` can include one or multiple values", {
  choices <- c("platform", "packages", "python", "external")

  # including "all" results in all info being selected
  expect_named(
    session_info(info = "all"),
    choices,
    ignore.order = TRUE
  )
  # even when other items are included
  expect_named(
    session_info(info = c("all", "platform")),
    choices,
    ignore.order = TRUE
  )

  # including "auto" (and not "all") results in auto info being included
  expect_gte(
    length(names(session_info(info = "auto"))),
    2
  )
  # even when other items are included
  expect_gte(
    length(names(session_info(info = c("auto", "platform")))),
    2
  )

  # with neither "all" or "auto", valid items are included
  expect_named(
    session_info(info = c("platform", "python", "external", "nonvalid-info")),
    c("platform", "python", "external"),
    ignore.order = TRUE
  )

})

test_that("broken dll", {
  if (.Platform$OS.type != "windows") { expect_true(TRUE); return() }
  skip_on_cran()

  ## To check this, we need a package with a dll.
  ## We need to install it into some temporary library, and then mess
  ## up the MD5 sum. We can use testhat itself to do this, as long as it
  ## has compiled code. We also run package_info() in another process,
  ## to avoid changing the current one.

  dir.create(lib <- tempfile())
  on.exit(unlink(lib, recursive = TRUE), add = TRUE)
  file.copy(system.file(package = "testthat"), lib, recursive = TRUE)

  md5file <- file.path(lib, "testthat", "MD5")
  if (!file.exists(md5file)) skip("Cannot test broken DLLs")
  l <- readLines(md5file)
  dllline <- grep("testthat.dll", l)[1]
  substr(l[dllline], 2, 5) <- "xxxx"
  writeLines(l, md5file)

  pi <- callr::r(
    function(lib) {
      library(testthat, lib.loc = lib)
      sessioninfo::package_info()
    },
    args = list(lib = lib),
    libpath = c(lib, .libPaths()),
    timeout = 10)

  expect_false(pi$md5ok[pi$package == "testthat"])
  expect_output(print(pi), "DLL MD5 mismatch, broken installation")
})

test_that("loaded & on-disk path mismatch", {
  skip_on_cran()

  ## Copy testthat to another library, load it from there, and then
  ## remove that lib from the library path.

  dir.create(lib <- tempfile())
  on.exit(unlink(lib, recursive = TRUE), add = TRUE)
  file.copy(system.file(package = "testthat"), lib, recursive = TRUE)

  pi <- callr::r(
    function(lib) {
      library(testthat, lib.loc = lib)
      .libPaths(.libPaths()[-1])
      sessioninfo::package_info()
    },
    args = list(lib = lib),
    libpath = c(lib, .libPaths()),
    timeout = 10
  )

  wh <- which(pi$package == "testthat")
  expect_false(pi$path[wh] == pi$loadedpath[wh])
  expect_output(print(pi), "Loaded and on-disk path mismatch")
})

test_that("loaded & on-disk version mismatch", {
  skip_on_cran()

  ## Copy testthat to another library and change the version, after
  ## loading it.

  dir.create(lib <- tempfile())
  on.exit(unlink(lib, recursive = TRUE), add = TRUE)
  file.copy(system.file(package = "testthat"), lib, recursive = TRUE)

  pi <- callr::r(
    function(lib) {
      library(testthat, lib.loc = lib)
      desc_file <- file.path(lib, "testthat", "DESCRIPTION")
      desc <- readLines(desc_file)
      desc <- sub("^Version:.*$", "Version: 0.0.1", desc)
      writeLines(desc, desc_file)

      binary_desc <- file.path(lib, "testthat", "Meta", "package.rds")
      if (file.exists(binary_desc)) {
        pkg_desc <- readRDS(binary_desc)
        desc <- as.list(pkg_desc$DESCRIPTION)
        desc$Version <- "0.0.1"
        pkg_desc$DESCRIPTION <- stats::setNames(as.character(desc), names(desc))
        saveRDS(pkg_desc, binary_desc)
      }
      sessioninfo::package_info()
    },
    args = list(lib = lib),
    libpath = c(lib, .libPaths()),
    timeout = 10
  )

  wh <- which(pi$package == "testthat")
  expect_false(pi$ondiskversion[wh] == pi$loadedversion[wh])
  expect_output(print(pi), "Loaded and on-disk version mismatch")

  # testthat is attached
  expect_output(print(pi), "testthat    * ")
  expect_output(print(pi), "Packages attached to the search path")
})

test_that("deleted package", {
  skip_on_cran()

  foo <- "fsdfgwetdhsdfhq4yqh"

  dir.create(lib <- tempfile())
  on.exit(unlink(lib, recursive = TRUE), add = TRUE)
  pkgfile <- normalizePath(paste0("fixtures/", foo, "_0.0.0.9000.tar.gz"))
  install.packages(pkgfile, lib = lib, repos = NULL, type = "source",
                   quiet = TRUE)

  pis <- callr::r(
    function(lib, foo) {
      library(foo, character.only = TRUE, lib.loc = lib)
      unlink(file.path(lib, foo), recursive = TRUE)
      list(
        sessioninfo::session_info(),
        sessioninfo::session_info(pkgs = foo)
      )
    },
    args = list(lib = lib, foo = foo),
    libpath = c(lib, .libPaths()),
    timeout = 10,
    error = "stack"
  )

  expect_true(is.list(pis))
  expect_equal(length(pis), 2)

  for (i in seq_along(pis)) {
    pi <- pis[[i]]$packages
    wh <- which(pi$package == foo)
    expect_equal(pi$ondiskversion[wh], NA_character_)
    expect_equal(pi$path[wh], NA_character_)
    expect_equal(pi$date[wh], NA_character_)
    expect_equal(pi$source[wh], NA_character_)
  }
})

#' Information about related software
#'
#' @details
#' Note that calling this function will attempt to load the tcltk and
#' grDevices packages.
#'
#' @return A list with elements:
#'   * `cairo`: The cairo version string.
#'   * `libpng`: The png version string.
#'   * `jpeg`: The jpeg version string.
#'   * `tiff`: The tiff library and version string used.
#'   * `tcl`: The tcl version string.
#'   * `curl`: The curl version string.
#'   * `zlib`: The zlib version string.
#'   * `bzlib`: The zlib version string.
#'   * `xz`: The zlib version string.
#'   * `PCRE`: The Perl Compatible Regular Expressions (PCRE) version string.
#'   * `ICU`: The International Components for Unicode (ICU) version string.
#'   * `TRE`: The TRE version string.
#'   * `iconv`: The iconv version string.
#'   * `readline`: The readline version string.
#'   * `BLAS`: The path with the implementation of BLAS in use.
#'   * `LAPACK`: The path with the implementation of LAPACK in use.
#'
#'
#' @seealso Similar functions and objects in the base packages:
#'   [utils::sessionInfo()], [base::extSoftVersion], [tcltk::tclVersion()]
#'   [base::La_library], [base::La_version()], [base::libcurlVersion()].
#'
#' @export
#' @examplesIf FALSE
#' external_info()

external_info <- function() {
  ex <- c(
    get_grsoft_version(),
    tcl = get_tcl_version(),
    curl = libcurlVersion(),
    extSoftVersion()
  )
  ex["lapack"] <- get_la_library()
  ex["lapack_version"] <- get_la_version()
  names(ex) <- gsub("^lib", "", names(ex))

  structure(as.list(ex), class = c("external_info", "list"))
}

get_tcl_version <- function() {
  tryCatch(
    suppressWarnings(tcltk::tclVersion()),
    error = function(err) ""
  )
}

get_grsoft_version <- function() {
  grDevices::grSoftVersion()
}

get_la_library <- function() {
  tryCatch(base::La_library(), error = function(err) NA_character_)
}

get_la_version <- function() {
  tryCatch(base::La_version(), error = function(err) NA_character_)
}

#' @export

format.external_info <- function(x, ...) {
  df <- data.frame(
    setting = names(x),
    value = unlist(x),
    stringsAsFactors = FALSE
  )
  format_df(df)
}

#' @export

print.external_info <- function(x, ...) {
  cat(format(x, ...), sep = "\n")
}

#' @export

as.character.external_info <- function(x, ...) {
  old <- options(cli.num_colors = 1)
  on.exit(options(old), add = TRUE)
  format(x, ...)
}

#' Information about the current platform
#'
#' @return A list with elements:
#'   * `version`: the R version string.
#'   * `os`: the OS name in human readable format, see [os_name()].
#'   * `system`: CPU, and machine readable OS name, separated by a comma.
#'   * `ui`: the user interface, e.g. `Rgui`, `RTerm`, etc. see `GUI`
#'     in [base::.Platform].
#'   * `hostname`: the name of the machine known on the network, see
#'     `nodename` in [base::Sys.info()]. For privacy, it is only included
#'     if the `sessioninfo.include_hostname` option is set to `TRUE`.
#'   * `language`: The current language setting. The `LANGUAGE` environment
#'     variable, if set, or `(EN)` if unset.
#'   * `collate`: Collation rule, from the current locale.
#'   * `ctype`: Native character encoding, from the current locale.
#'   * `tz`: The current time zone.
#'   * `date`: The current date.
#'   * `rstudio`: RStudio format string, only added in RStudio.
#'   * `pandoc`: pandoc version and path
#'   * `quarto`: quarto version and path
#'
#' @seealso Similar functions and objects in the base packages:
#'   [base::R.version.string], [utils::sessionInfo()], [base::version],
#'   [base::.Platform], [base::Sys.getlocale()], [base::Sys.timezone()].
#'
#' @export
#' @examplesIf FALSE
#' platform_info()

platform_info <- function() {
  include_hostname <- isTRUE(getOption("sessioninfo.include_hostname"))
  as_platform_info(drop_null(list(
    version = R.version.string,
    os = os_name(),
    system = version$system,
    hostname = if (include_hostname) {
      Sys.info()[["nodename"]]
    },
    ui = .Platform$GUI,
    language = Sys.getenv("LANGUAGE", "(EN)"),
    collate = Sys.getlocale("LC_COLLATE"),
    ctype = Sys.getlocale("LC_CTYPE"),
    tz = Sys.timezone(),
    date = format(Sys.Date()),
    rstudio = get_rstudio_version(),
    pandoc = get_pandoc_version(),
    quarto = get_quarto_version()
  )))
}

get_rstudio_version <- function() {
  tryCatch(
    {
      ver <- get("RStudio.Version", "tools:rstudio")()
      paste0(
        ver$long_version %||% ver$version,
        if (!is.null(ver$release_name)) paste0(" ", ver$release_name),
        " (",
        ver$mode,
        ")"
      )
    },
    error = function(e) NULL
  )
}

get_pandoc_version <- function() {
  if (isNamespaceLoaded("rmarkdown")) {
    ver <- rmarkdown::find_pandoc()
    if (is.null(ver$dir)) {
      "NA (via rmarkdown)"
    } else {
      paste0(ver$version, " @ ", ver$dir, "/ (via rmarkdown)")
    }
  } else {
    path <- Sys.which("pandoc")
    if (path == "") {
      "NA"
    } else {
      ver <- parse_pandoc_version(path)
      paste0(ver, " @ ", path)
    }
  }
}

parse_pandoc_version <- function(path) {
  tryCatch(
    {
      out <- system2(path, "--version", stdout = TRUE)[1]
      last(strsplit(out, " ", fixed = TRUE)[[1]])
    },
    error = function(e) "NA"
  )
}

get_quarto_version <- function() {
  path <- Sys.which("quarto")
  if (path == "") {
    "NA"
  } else {
    tmpdir <- tempfile("quarto-")
    dir.create(tmpdir, recursive = TRUE, showWarnings = FALSE)
    on.exit(unlink(tmpdir, recursive = TRUE), add = TRUE)

    oldtmpdir <- Sys.getenv("TMPDIR", unset = NA)
    on.exit(
      {
        if (is.na(oldtmpdir)) {
          Sys.unsetenv("TMPDIR")
        } else {
          Sys.setenv(TMPDIR = oldtmpdir)
        }
      },
      add = TRUE
    )

    Sys.setenv(TMPDIR = tmpdir)
    ver <- system2("quarto", "-V", stdout = TRUE)[1]
    paste0(ver, " @ ", path)
  }
}

#' @export

format.platform_info <- function(x, ...) {
  df <- data.frame(
    setting = names(x),
    value = unlist(x),
    stringsAsFactors = FALSE
  )
  format_df(df)
}

#' @export

print.platform_info <- function(x, ...) {
  cat(format(x, ...), sep = "\n")
}

#' @export

as.character.platform_info <- function(x, ...) {
  old <- options(cli.num_colors = 1)
  on.exit(options(old), add = TRUE)
  format(x, ...)
}

#' @export

c.platform_info <- function(...) {
  as_platform_info(NextMethod())
}

as_platform_info <- function(x) {
  stopifnot(is.list(x))
  class(x) <- c("platform_info", "list")
  x
}

test_that("platform_info", {
  withr::local_options(sessioninfo.include_hostname = FALSE)
  pi <- platform_info()
  nms <- c(
    "version",
    "os",
    "system",
    "hostname",
    "ui",
    "language",
    "collate",
    "ctype",
    "tz",
    "date",
    "pandoc",
    "quarto"
  )
  expect_equal(names(pi), setdiff(nms, "hostname"))

  withr::local_options(sessioninfo.include_hostname = TRUE)
  pi <- platform_info()
  expect_equal(names(pi), nms)

  ## This can be a variety of strings, e.g. "R Under development"
  expect_match(pi$version, "R ")
  expect_true(is_string(pi$os))
  expect_true(is_string(pi$system) && grepl(",", pi$system))
  expect_true(is_string(pi$ui))
  expect_true(is_string(pi$language))
  expect_true(is_string(pi$tz) || identical(pi$tz, NA_character_))
  expect_true(is_string(pi$date))
  expect_equal(pi$date, as.character(as.Date(pi$date)))
})

test_that("print.platform_info", {
  expect_output(print(platform_info()), "setting[ ]+value")
})

test_that("print.platform_info ignores max.print", {
  pi <- platform_info()
  withr::local_options(list(max.print = 1))
  out <- capture_output(print(pi))
  out <- tail(strsplit(out, split = "\r?\n")[[1]], -1)
  expect_length(out, length(pi))
})

test_that("get_quarto_version", {
  local_mocked_bindings(Sys.which = function(...) "")
  expect_snapshot(get_quarto_version())

  local_mocked_bindings(Sys.which = function(...) "/path/to/quarto")
  local_mocked_bindings(system2 = function(...) "1.3.450")
  expect_snapshot(get_quarto_version())
})

#' @importFrom utils packageVersion

loaded_packages <- function() {
  get_package_info(loadedNamespaces())
}

attached_packages <- function() {
  packages <- intersect(
    loadedNamespaces(),
    sub("^package:", "", search())
  )
  get_package_info(packages)
}

installed_packages <- function() {
  pkgs <- rownames(utils::installed.packages(noCache = TRUE))
  dependent_packages(pkgs, dependencies = FALSE)
}

get_package_info <- function(packages) {
  ## 'base' is special, because getNamespaceInfo does not work on it.
  ## Luckily, the path for 'base' is just system.file()

  spackageVersion <- function(pkg) {
    ## Error may happen if the package was loaded, and then removed from
    ## the disk. In this case we'll have NA
    tryCatch(
      as.character(packageVersion(pkg, lib.loc = .libPaths())),
      error = function(e) NA_character_
    )
  }

  packages <- setdiff(packages, "base")

  loadedversion <- vapply(packages, getNamespaceVersion, "")
  ondiskversion <- vapply(packages, spackageVersion, "")

  path <- vapply(
    packages,
    function(p) system.file(package = p, lib.loc = .libPaths()),
    character(1)
  )
  ## If we can't fine the package on disk, have NA instead of ""
  path[path == ""] <- NA_character_
  loadedpath <- vapply(packages, getNamespaceInfo, "", which = "path")

  attached <- paste0("package:", packages) %in% search()

  res <- data.frame(
    package = c(packages, "base"),
    ondiskversion = c(ondiskversion, spackageVersion("base")),
    loadedversion = c(loadedversion, getNamespaceVersion("base")),
    path = c(path, system.file()),
    loadedpath = c(loadedpath, NA_character_),
    attached = c(attached, TRUE),
    stringsAsFactors = FALSE,
    row.names = NULL
  )

  res <- res[match(sort_ci(res$package), res$package), ]

  row.names(res) <- NULL
  res
}

test_that("loaded_packages", {
  lp <- loaded_packages()
  expect_equal(sort_ci(lp$package), sort_ci(loadedNamespaces()))
  expect_identical(
    gsub("-", ".", lp$loadedversion),
    unlist(lapply(lp$package, function(x) as.character(packageVersion(x))))
  )
  expect_true(all(file.exists(lp$path)))
  expect_equal(
    paste0("package:", lp$package) %in% search(),
    lp$attached
  )
})

should_show_python <- function(pkgs) {
  "reticulate" %in%
    pkgs ||
    (isNamespaceLoaded("reticulate") &&
      reticulate::py_available(initialize = FALSE))
}

#' Python configuration
#'
#' @return
#' Returns a [reticulate::py_config] object, which also has the
#' `python_info` class. It is a named list of values.
#'
#' If reticulate is not installed or Python is not configured,
#' then it return a `python_info` object that is a character vector, and
#' it does not have a `py_config` class.
#'
#' @export
#' @examplesIf FALSE
#' python_info()
#' session_info(info = "all")

python_info <- function() {
  conf <- if (
    isNamespaceLoaded("reticulate") &&
      reticulate::py_available(initialize = FALSE)
  ) {
    tryCatch(
      reticulate::py_config(),
      error = function(err) {
        c("`reticulate::py_config()` failed with error:", conditionMessage(err))
      }
    )
  } else {
    "Python is not available"
  }

  class(conf) <- unique(c("python_info", class(conf), "list"))
  conf
}

#' @export

format.python_info <- function(x, ...) {
  x <- NextMethod()
  paste0(" ", unlist(strsplit(x, "\n", fixed = TRUE)))
}

#' @export

as.character.python_info <- function(x, ...) {
  old <- options(cli.num_colors = 1)
  on.exit(options(old), add = TRUE)
  format(x, ...)
}

#' @export

print.python_info <- function(x, ...) {
  cat(format(x, ...), sep = "\n")
}

dependent_packages <- function(pkgs, dependencies) {
  ideps <- interpret_dependencies(dependencies)

  pkgs <- find_deps(pkgs, utils::installed.packages(), ideps[[1]], ideps[[2]])
  desc <- lapply(pkgs, pkg_desc)

  loaded_pkgs <- pkgs %in% setdiff(loadedNamespaces(), "base")
  ondiskversion <- vapply(
    desc,
    function(x) x$Version %||% NA_character_,
    character(1)
  )
  loadedversion <- rep(NA_character_, length(pkgs))
  loadedversion[loaded_pkgs] <- vapply(
    pkgs[loaded_pkgs],
    getNamespaceVersion,
    ""
  )
  loadedpath <- rep(NA_character_, length(pkgs))
  loadedpath[loaded_pkgs] <-
    vapply(pkgs[loaded_pkgs], getNamespaceInfo, "", which = "path")
  res <- data.frame(
    package = pkgs,
    ondiskversion = ondiskversion,
    loadedversion = loadedversion,
    path = vapply(desc, pkg_path_disk, character(1)),
    loadedpath = loadedpath,
    attached = paste0("package:", pkgs) %in% search(),
    stringsAsFactors = FALSE,
    row.names = NULL
  )

  res <- res[match(sort_ci(res$package), res$package), ]

  row.names(res) <- NULL
  res
}

pkg_path_disk <- function(desc) {
  if (is.null(desc)) {
    NA_character_
  } else {
    system.file(package = desc$Package, lib.loc = .libPaths())
  }
}

find_deps <- function(
  pkgs,
  available = utils::available.packages(),
  top_dep = c(dep_types_hard(), "Suggests"),
  rec_dep = dep_types_hard(),
  include_pkgs = TRUE
) {
  if (length(pkgs) == 0 || identical(top_dep, FALSE)) return(character())

  if (length(top_dep) > 0) {
    top <- tools::package_dependencies(pkgs, db = available, which = top_dep)
    top_flat <- unlist(top, use.names = FALSE)
  } else {
    top_flat <- character()
  }

  if (length(rec_dep) != 0 && length(top_flat) > 0) {
    rec <- tools::package_dependencies(
      top_flat,
      db = available,
      which = rec_dep,
      recursive = TRUE
    )
    rec_flat <- unlist(rec, use.names = FALSE)
  } else {
    rec_flat <- character()
  }

  unique(c(if (include_pkgs) pkgs, top_flat, rec_flat))
}

dep_types_hard <- function() c("Depends", "Imports", "LinkingTo")
dep_types_soft <- function() c("Suggests", "Enhances")
dep_types <- function() c(dep_types_hard(), dep_types_soft())

is_na_scalar <- function(x) length(x) == 1 && is.na(x)

interpret_dependencies <- function(dp) {
  hard <- dep_types_hard()

  if (isTRUE(dp)) {
    list(c(hard, "Suggests"), hard)
  } else if (identical(dp, FALSE)) {
    list(character(), character())
  } else if (is_na_scalar(dp)) {
    list(hard, hard)
  } else {
    list(dp, dp)
  }
}

test_that("dependent_packages", {
  ins <- readRDS("fixtures/installed.rda")
  dep <- readRDS("fixtures/devtools-deps.rda")
  alldsc <- readRDS("fixtures/descs.rda")

  local_mocked_bindings(installed.packages = function() ins, .package = "utils")
  local_mocked_bindings(pkg_desc = function(x) alldsc[[x]])
  local_mocked_bindings(loadedNamespaces = function() ins)

  local_mocked_bindings(getNamespaceVersion = function(x) alldsc[[x]]$Version)
  local_mocked_bindings(
    search = function() paste0("package:", dep$package[dep$attached])
  )
  local_mocked_bindings(getNamespaceInfo = function(x, ...) alldsc[[x]]$Version)

  exp <- dep[, setdiff(colnames(dep), c("path", "loadedpath"))]
  tec <- dependent_packages("devtools", NA)
  tec <- tec[, setdiff(colnames(tec), c("path", "loadedpath"))]
  expect_equal(exp, tec)
})

test_that("pkg_path_disk", {
  p1 <- pkg_path_disk(utils::packageDescription("stats"))
  expect_equal(
    read.dcf(file.path(p1, "DESCRIPTION"))[, "Package"],
    c(Package = "stats")
  )
})

test_that("find_deps", {
  ins <- readRDS("fixtures/installed.rda")
  expect_equal(
    sort_ci(find_deps("devtools", ins)),
    sort_ci(
      c("devtools", "callr", "httr", "utils", "tools", "methods", "memoise",
        "whisker", "digest", "rstudioapi", "jsonlite", "stats", "git2r",
        "withr", "pkgbuild", "pkgload", "curl", "crayon", "testthat",
        "BiocInstaller", "Rcpp", "MASS", "rmarkdown", "knitr", "hunspell",
        "lintr", "bitops", "roxygen2", "evaluate", "rversions", "covr",
        "gmailr", "processx", "R6", "assertthat", "debugme", "grDevices",
        "graphics", "mime", "openssl", "desc", "rprojroot", "backports",
        "praise", "magrittr", "xml2", "BH", "yaml", "htmltools", "caTools",
        "base64enc", "stringr", "highr", "markdown", "stringi", "rex",
        "codetools", "stringdist", "xmlparsedata", "lazyeval", "parallel",
        "brew", "commonmark")
    )
  )

  ## An edge case
  expect_equal(
    find_deps("foobar", top_dep = character(), rec_dep = character()),
    "foobar"
  )
})

test_that("find_deps", {
  ins <- readRDS("fixtures/installed.rda")
  expect_equal(
    find_deps("devtools", ins, top_dep = FALSE),
    character()
  )
})

test_that("dep_types", {
  skip_on_cran()
  withr::local_options(repos = c(CRAN = "https://cloud.r-project.org"))
  expect_silent(
    tools::package_dependencies("sessioninfo", which = dep_types())
  )
})

test_that("interpret_dependencies", {
  skip_on_cran()
  withr::local_options(repos = c(CRAN = "https://cloud.r-project.org"))
  expect_silent(
    tools::package_dependencies(
      "sessioninfo",
      which = interpret_dependencies(TRUE)[[1]]
    )
  )
  expect_silent(
    tools::package_dependencies(
      "sessioninfo",
      which = interpret_dependencies(TRUE)[[2]]
    )
  )

  expect_equal(
    interpret_dependencies(FALSE),
    list(character(), character())
  )

  expect_silent(
    tools::package_dependencies(
      "sessioninfo",
      which = interpret_dependencies(NA)[[1]]
    )
  )
  expect_silent(
    tools::package_dependencies(
      "sessioninfo",
      which = interpret_dependencies(NA)[[2]]
    )
  )

  expect_equal(
    interpret_dependencies("Depends"),
    list("Depends", "Depends")
  )
})

`%||%` <- function(l, r) if (is.null(l)) r else l

sort_ci <- function(x) {
  old <- Sys.getlocale("LC_COLLATE")
  on.exit(Sys.setlocale("LC_COLLATE", old), add = TRUE)
  Sys.setlocale("LC_COLLATE", "C")
  x[order(tolower(x), x)]
}

is_string <- function(x) {
  is.character(x) && length(x) == 1 && !is.na(x)
}

is_flag <- function(x) {
  is.logical(x) && length(x) == 1 && !is.na(x)
}

order_by_name <- function(x) {
  if (!length(x)) {
    x
  } else if (is.null(names(x))) {
    stop("Cannot order by name, no names")
  } else {
    x[order(names(x))]
  }
}

drop_null <- function(x) {
  x[!vapply(x, is.null, logical(1))]
}

last <- function(x) {
  x[length(x)]
}

empty <- function(x) {
  grepl("^\\s*$", x)
}

str_trim <- function(x, width) {
  stopifnot(width >= 2)
  if (nchar(x) <= width) {
    x
  } else {
    paste0(substr(x, 1, width - 1), "~")
  }
}

test_that("sort_ci", {
  expect_equal(
    sort_ci(c("ant", "\u00a0nt", "bottom")),
    c("ant", "bottom", "\u00a0nt")
  )
})

test_that("order_by_name", {
  expect_equal(order_by_name(list()), list())

  expect_snapshot(error = TRUE, order_by_name(c("foo", "bar")))
})

#' Compare session information from two sources
#'
#' @param old,new A `session_info` object (the return value of
#' [session_info()]), or a pointer to [session_info()] output. See details
#' below.
#' @param packages How to compare the package info for `old` and `new`:
#' * `"diff"`: line diffs, in the style of `diff -u`
#' * `"merge"`: merge the information into one data frame, with one row per
#'   package. Only package `version` and `source` are compared.
#' @param ... Passed to any new [session_info()] calls.
#'
#' @details
#' Various way to specify `old` and `new`:
#' * A `session_info` object.
#' * `"local"` runs [session_info()] in the current
#'   session, and uses its output.
#' * `"clipboard"` takes the session info from the system clipboard.
#'   If the clipboard contains a URL, it is followed to download the
#'   session info.
#' * The URL where you inspect the results for a GitHub Actions job.
#'   Typically has this form:
#'   ```
#'   https://github.com/OWNER/REPO/actions/runs/RUN_ID/jobs/HTML_ID
#'   ```
#'   Internally, this URL is parsed so we can look up the job id, get the
#'   log file, and extract session info.
#' * Any other URL starting with `http://` or `https://`. `session_diff()`
#'   searches the HTML (or text) page for the session info header to find the
#'   session info.
#'
#'
#' @export
#' @examplesIf FALSE
#' session_diff()

session_diff <- function(old = "local", new = "clipboard",
                         packages = c("diff", "merge"), ...) {

  packages <- match.arg(packages)

  oldname <- get_symbol_name(substitute(old))
  newname <- get_symbol_name(substitute(new))

  old <- get_session_info(old, oldname %||% "old", ...)
  new <- get_session_info(new, newname %||% "new", ...)

  ret <- list(
    old = old,
    new = new,
    diff = session_diff_text(old$text, new$text, packages)
  )

  class(ret) <- c("session_diff", "list")
  ret
}

#' @export

format.session_diff <- function(x, ...) {
  c(
    cli::style_bold(paste0("--- ", substr(x$old$name, 1, 78))),
    cli::style_bold(paste0("+++ ", substr(x$new$name, 1, 78))),
    format(x$diff, context = Inf)
  )
}

#' @export

print.session_diff <- function(x, ...) {
  writeLines(format(x, ...))
}

get_session_info <- function(src, name = NULL, ...) {
  si <- if (is_string(src) == 1 && src == "local") {
    get_session_info_local(...)
  } else if (is_string(src) == 1 && src == "clipboard") {
    get_session_info_clipboard()
  } else if (is_string(src) && is_gha_url(src)) {
    get_session_info_gha(src)
  } else if (is_string(src) && grepl("https?://", src)) {
    get_session_info_url(src)
  } else {
    get_session_info_literal(src)
  }
  if (is.null(si$name)) si$name <- name
  si
}

get_session_info_local <- function(...) {
  si <- session_info(...)
  old <- options(cli.num_colors = 1)
  on.exit(options(old), add = TRUE)
  list(arg = "local", si = si, text = format(si))
}

get_session_info_clipboard <- function() {
  cnt <- clipboard_read()
  if (is_string(cnt) && cnt == "clipboard") {
    si <- list(arg = "clipboard", si = cnt, text = cnt)
  } else {
    si <- get_session_info(cnt)
  }
  si$arg <- "<clipboard>"
  si
}

get_session_info_url <- function(url) {
  tmp <- tempfile("session-diff-")
  on.exit(unlink(tmp), add = TRUE)
  suppressWarnings(utils::download.file(url, tmp, quiet = TRUE, mode = "wb"))
  html <- readLines(url, warn = FALSE, encoding = "UTF-8")
  find_session_info_in_html(url, html)
}

find_session_info_in_html <- function(url, lines) {
  purl <- parse_url(url)
  re_start <- "[-=\u2500\u2550][ ]Session info[ ]"
  cand <- grep(re_start, lines)
  if (length(cand) == 0) stop("Cannot find session info at '", url, "'.")

  # in the new GH HTML the whole comment is a data field that we extract
  if (any(grepl("\\n", lines, fixed = TRUE))) {
    lines <- gsub("\\r", "", lines, fixed = TRUE)
    lines <- unlist(strsplit(lines, "\\n", fixed = TRUE))
    cand <- grep(re_start, lines)
    if (length(cand) == 0) stop("Cannot find session info at '", url, "'.")
  }

  # check if the URL has an anchor and that the anchor exists in HTML
  # if yes, then we "skip" there
  if (purl$anchor != "") {
    anch <- which(
      grepl(paste0(" id=\"", purl$anchor, "\""), lines, fixed = TRUE) |
      grepl(paste0(" id='", purl$anchor, "'"), lines, fixed = TRUE) |
      grepl(paste0(" id = \"", purl$anchor, "\""), lines, fixed = TRUE) |
      grepl(paste0(" id = '", purl$anchor, "'"), lines, fixed = TRUE)
    )[1]
    if (!is.na(anch) && any(anch < cand)) {
      lines <- lines[anch:length(lines)]
      cand <- grep(re_start, lines)
    } else {
      url <- sub(paste0("#", purl$anchor), "", url)
    }
  }

  lines <- lines[cand[1]:length(lines)]
  lines[1] <- sub(paste0("^.*(", re_start, ")"), "\\1", lines[1])

  grepl_end <- function(lines) {
    grepl("^(#&gt;)?[ ]*\\[[0-9]\\] ", lines) |
      grepl("^(#&gt;)?[ ]*[-\u2500]+$", lines)
  }

  end <- which(grepl_end(lines))[1]

  if (is.na(end)) stop("Cannot parse session info from '", url, "'.")
  while (end < length(lines) && grepl_end(lines[end + 1])) {
    end <- end + 1
  }

  si <- get_session_info_literal(lines[1:end])
  si$arg <- url
  si$name <- url
  si
}

parse_url <- function (url) {
  re_url <- paste0(
    "^(?<protocol>[a-zA-Z0-9]+)://",
    "(?:(?<username>[^@/:]+)(?::(?<password>[^@/]+))?@)?",
    "(?<host>[^/]+)",
    "(?<path>[^#]*)",
    "#?(?<anchor>.*)$"
  )
  re_match(url, re_url)$groups
}

get_session_info_literal <- function(si) {
  if (inherits(si, "session_info")) {
    old <- options(cli.num_colors = 1)
    on.exit(options(old), add = TRUE)
    list(arg = si, si = si, text = format(si))

  } else if (is.character(si)) {
    # in case it has ANSI sequences
    text <- cli::ansi_strip(si)

    # Might be a single string
    text <- unlist(strsplitx(text, "\n", fixed = TRUE))

    # reprex has the knitr output prefix, remove it
    # order is important here
    text <- sub("^#&gt;[ ]?", "", text)
    text <- sub("^#[>#]?[ ]?", "", text)

    check_session_info(text)
    list(arg = si, si = si, text = text)

  } else {
    stop("Could not interpret a `", class(si), "` as a session info.")
  }
}

# strsplit("", "\n") -> character(), but it should be the empty string,
# so we fix this.

strsplitx <- function(...) {
  lapply(strsplit(...), paste0, "")
}

check_session_info <- function(x) {
  if (!any(grepl("[-=\u2500\u2550] Session info", x))) {
    warning("This does not look like a session info: '", beginning(x), "'.")
  }
}

beginning <- function(x) {
  x123 <- utils::head(unlist(strsplit(x, "\n", fixed = TRUE)), 3)
  trimws(substr(paste0(x123, sep = "\n"), 1, 100))
}

session_diff_text <- function(old, new, packages = c("diff", "merge")) {
  packages <- match.arg(packages)

  old <- enc2utf8(old)
  new <- enc2utf8(new)

  old <- diff_drop_empty(old)
  new <- diff_drop_empty(new)

  old <- diff_no_date(old)
  new <- diff_no_date(new)

  min <- diff_min_line(c(old, new))
  old <- diff_fix_lines(old, min)
  new <- diff_fix_lines(new, min)

  package_fixup_fun <- switch(
    packages,
    # expands package info with fewer columns to match one with more
    diff = expand_diff_text,
    # full-joins package info to concentrate each diff in a single row
    merge = merge_packages
  )
  # do not error, in case we cannot parse sessioninfo output
  suppressWarnings(tryCatch({
    exp <- package_fixup_fun(old, new)
    old <- exp$old
    new <- exp$new
  }, error = function(e) NULL))

  old2 <- gsub("\\s+", " ", old)
  new2 <- gsub("\\s+", " ", new)

  diff <- cli::diff_chr(old2, new2)
  diff$old <- old
  diff$new <- new
  diff
}

# drop leading and trailing empty lines

diff_drop_empty <- function(x) {
  len <- length(x)
  if (len == 0) return(x)

  empty <- rle(grepl("^\\s*$", x))
  pre <- if (empty$values[1]) {
    1:empty$lengths[1]
  }
  post <- if (utils::tail(empty$values, 1)) {
    (len - utils::tail(empty$lengths, 1) + 1):len
  }
  del <- as.integer(c(pre, post))
  if (length(del)) x <- x[-del]

  x
}

# Drop the first `date` line

diff_no_date <- function(x) {
  date <- grep("^[ ]*date[ ]+[0-9][0-9][0-9][0-9]-", x)
  if (length(date) > 0) {
    x <- x[-date[1]]
  }
  x
}

# Calculate the minimum width of the header lines

diff_min_line <- function(x) {
  lines <- c(
    grep("[-\u2500][-\u2500][-\u2500]$", x),
    grep("[=\u2550][=\u2550][=\u2550]$", x)
  )
  min(c(80, cli::utf8_nchar(x[lines], "width")))
}

diff_fix_lines <- function(x, w) {
  slines <- grepl("[-\u2500]+$", x)
  dlines <- grepl("[=\u2550]+$", x)
  x[slines] <- gsub("[-\u2500]", cli::symbol$line, x[slines])
  x[dlines] <- gsub("[=\u2550]", cli::symbol$double_line, x[dlines])
  x[slines] <- substr(x[slines], 1, w)
  x[dlines] <- substr(x[dlines], 1, w)
  x
}

expand_diff_text <- function(old, new) {
  opkgs <- parse_pkgs(old)
  npkgs <- parse_pkgs(new)

  if (is.null(opkgs) || is.null(opkgs)) return(list(old = old, new = new))

  # Add the "!" column if needed
  if ("!" %in% names(opkgs$pkgs) || "!" %in% names(npkgs$pkgs)) {
    if (! "!" %in% names(opkgs$pkgs)) {
      opkgs$pkgs <- cbind("!" = "", opkgs$pkgs, stringsAsFactors = FALSE)
    }
    if (! "!" %in% names(npkgs$pkgs)) {
      npkgs$pkgs <- cbind("!" = "", npkgs$pkgs, stringsAsFactors = FALSE)
    }
  }

  # If the column names differ, we keep it as is
  onms <- names(opkgs$pkgs)
  nnms <- names(npkgs$pkgs)
  if (length(onms) != length(nnms) || any(onms != nnms)) {
    return(list(old = old, new = new))
  }

  cmn <- rbind(opkgs$pkgs, npkgs$pkgs)
  oldopts <- options(cli.num_colors = 1)
  on.exit(options(oldopts), add = TRUE)
  fmt <- format_df(cmn)

  oend <- opkgs$end - opkgs$begin + 1L
  nend <- oend + npkgs$end - npkgs$begin
  fmt_old <- c(fmt[1], fmt[2:oend])
  fmt_new <- c(fmt[1], fmt[(oend+1):(nend)])

  old <- insert_instead(old, opkgs$begin, opkgs$end, fmt_old)
  new <- insert_instead(new, npkgs$begin, npkgs$end, fmt_new)

  list(old = old, new = new)
}

merge_packages <- function(old, new) {
  opkgs <- parse_pkgs(old)
  npkgs <- parse_pkgs(new)

  if (is.null(opkgs) || is.null(opkgs)) return(list(old = old, new = new))

  names_to_keep <- c("package", "version", "source")
  opkgs$pkgs <- opkgs$pkgs[names_to_keep]
  npkgs$pkgs <- npkgs$pkgs[names_to_keep]
  names(opkgs$pkgs) <- c("package", "old_version", "old_source")
  names(npkgs$pkgs) <- c("package", "new_version", "new_source")

  merge_res <- merge(opkgs$pkgs, npkgs$pkgs, all = TRUE)
  merge_res <- with(merge_res,
       data.frame(
         package = package,
         "v!=" = mark(is_different(old_version, new_version)),
         old_version = old_version,
         new_version = new_version,
         "s!=" = mark(is_different(old_source, new_source)),
         old_source = old_source,
         new_source = new_source,
         stringsAsFactors = FALSE,
         row.names = NULL,
         check.names = FALSE
       )
  )

  oldopts <- options(cli.num_colors = 1)
  on.exit(options(oldopts), add = TRUE)
  fmt <- format_df(merge_res)

  old <- insert_instead(old, opkgs$begin, opkgs$end, fmt)
  new <- insert_instead(new, npkgs$begin, npkgs$end, fmt)

  list(old = old, new = new)
}

is_different <- function(x, y) {
  x_na <- is.na(x)
  y_na <- is.na(y)
  no_na <- !x_na & !y_na
  xor(x_na, y_na) | (no_na & (x != y))
}

mark <- function(x, mark = ">>") {
  ifelse(x, mark, "")
}

insert_instead <- function(orig, from, to, new) {
  pre <- if (from > 1) orig[1:(from-1)]
  pst <- if (to < length(orig)) orig[(to+1):length(orig)]
  c(pre, new, pst)
}

parse_pkgs <- function(lines) {
  begin <- grep("^[-\u2500] Packages ", lines) + 1

  # back out if no Packages header
  if (length(begin) != 1 || length(begin) > length(lines)) return(NULL)

  # now find the end
  end <- begin + grep(
    "^\\s*[!a-zA-Z]",
    lines[begin:length(lines)],
    invert = TRUE,
    perl = TRUE
  )[1] - 2
  if (is.na(end)) end <- length(lines)

  pkgs <- parse_pkgs_section(lines[begin:end])

  list(begin = begin, end = end, pkgs = pkgs)
}

parse_pkgs_section <- function(lines) {
  lines[1] <- sub(" date  ", " date (UTC) ", fixed = TRUE, lines[1])
  hdr <- sub("date (UTC)", "date-(UTC)", fixed = TRUE, lines[1])
  wth <- find_word_lengths(hdr)
  wth[length(wth)] <- max(nchar(lines))
  df <- utils::read.fwf(textConnection(lines), widths = wth)
  df[] <- lapply(df, trimws)
  names(df) <- as.character(df[1,])
  df <- df[-1, , drop = FALSE]
  rownames(df) <- NULL
  df
}

find_word_lengths <- function(x) {
  # add a dummy word to the end for simplicity
  tmp <- paste0(gsub("[^\\s]", "X", x, perl = TRUE), " X")

  # word & ws lengths, but first absorb leading space into the first word
  ltr <- strsplit(tmp, "")[[1]]
  rl <- rle(ltr)
  if (ltr[1] == " ") {
    rl$lengths[2] <- rl$lengths[2] + rl$lengths[1]
    rl$lengths <- rl$lengths[-1]
    rl$values <- rl$values[-1]
  }

  # positions of "X" and " " parts
  pos <- cumsum(c(1, rl$lengths))

  # lengths of words, this drops the dummy word
  wpos <- which(rl$values == "X")
  pos[wpos[-1]] - pos[wpos[-length(wpos)]]
}

get_symbol_name <- function(x) {
  if (is.symbol(x)) {
    as.character(x)
  } else if (is_string(x)) {
    x
  }
}

test_that("session_diff", {
  lines1 <- readLines(test_path("fixtures", "lines1.txt"))
  lines2 <- readLines(test_path("fixtures", "lines2.txt"))
  sd <- session_diff(lines1, lines2)
  expect_equal(sd$old$si, lines1)
  expect_equal(sd$old$arg, lines1)
  expect_equal(sd$old$text, lines1)
  expect_equal(sd$new$si, lines2)
  expect_equal(sd$new$arg, lines2)
  expect_equal(sd$new$text, lines2)

  sd$old$si <- sd$old$arg <- sd$old$text <- NULL
  sd$new$si <- sd$new$arg <- sd$new$text <- NULL
  expect_snapshot(class(sd))
  expect_snapshot(unclass(sd))
})

test_that("format.session_diff", {
  # tested via print
  expect_true(TRUE)
})

test_that("print.session_diff", {
  lines1 <- readLines(test_path("fixtures", "lines1.txt"))
  lines2 <- readLines(test_path("fixtures", "lines2.txt"))
  expect_snapshot(print(session_diff(lines1, lines2)))
})

test_that("get_session_info 1", {
  arg <- NULL
  abort <- function(...) stop("test failure")
  local_mocked_bindings(get_session_info_local = function() abort)
  local_mocked_bindings(get_session_info_clipboard = function() abort)
  local_mocked_bindings(get_session_info_url = function() abort)
  local_mocked_bindings(get_session_info_literal = function() abort)

  local_mocked_bindings(
    get_session_info_local = function(...) arg <<- list(...)
  )
  get_session_info("local", pkgs = "foo")
  expect_equal(arg, list(pkgs = "foo"))

  arg <- NULL
  local_mocked_bindings(get_session_info_local = function() abort)
  local_mocked_bindings(
    get_session_info_clipboard = function(...) arg <<- list(...)
  )
  get_session_info("clipboard", pkgs = "foo")
  expect_equal(arg, list())

  arg <- NULL
  local_mocked_bindings(get_session_info_clipboard = function() abort)
  local_mocked_bindings(get_session_info_url = function(...) arg <<- list(...))
  get_session_info("https://acme.com", pkgs = "foo")
  expect_equal(arg, list("https://acme.com"))

  arg <- NULL
  local_mocked_bindings(get_session_info_url = function() abort)
  local_mocked_bindings(
    get_session_info_literal = function(...) arg <<- list(...)
  )
  get_session_info(c("foo", "bar"), pkgs = "foo")
  expect_equal(arg, list(c("foo", "bar")))
})

test_that("get_session_info_local", {
  expect_equal(
    get_session_info_local()$text,
    format(session_info())
  )
})

test_that("get_session_info_clipboard", {
  lines <- readLines(test_path("fixtures", "lines1.txt"))
  local_mocked_bindings(clipboard_read = function() lines)
  clp <- get_session_info_clipboard()
  expect_equal(clp$arg, "<clipboard>")
  expect_equal(
    clp$text,
    get_session_info_literal(lines)$text
  )

  local_mocked_bindings(clipboard_read = function() "clipboard")
  expect_equal(get_session_info_clipboard()$text, "clipboard")
})

test_that("get_session_info_url", {
  html <- readLines(
    gz <- gzfile(test_path("fixtures", "gh.html.gz")),
    encoding = "UTF-8"
  )
  close(gz)

  local_mocked_bindings(
    download.file = function(url, destfile, ...) writeLines(html, destfile),
    .package = "utils"
  )

  # Needs Internet connection
  skip_on_cran()
  url <- "https://github.com/r-lib/sessioninfo/issues/6"
  expect_equal(
    get_session_info_url(url),
    find_session_info_in_html(url, html)
  )
})

test_that("find_session_info_in_html", {
  # We skip this on old R, because it does not calculate the width
  # of the emojis properly, and that messes up the output of the
  # character vector of lines. We also cannot compare the UTF-8 text
  # on Windows.
  if (getRversion() < "4.0") skip("Needs R 4.0 at least")
  skip_on_os("windows")

  html <- readLines(
    gz <- gzfile(test_path("fixtures", "gh.html.gz")),
    encoding = "UTF-8"
  )
  close(gz)

  url <- "https://github.com/r-lib/sessioninfo/issues/6"
  expect_snapshot(
    find_session_info_in_html(url, html)$text
  )

  url2 <- paste0(url, "#issuecomment-937782988")
  expect_snapshot(
    find_session_info_in_html(url2, html)$text
  )

  url3 <- paste0(url, "#dfgdfgdfgdfgdfgdfg")
  expect_equal(
    find_session_info_in_html(url, html)$text,
    find_session_info_in_html(url3, html)$text
  )

  html <- html[
    !grepl("^(#&gt;)?[ ]*\\[[0-9]\\] ", html) &
    !grepl("^(#&gt;)?[ ]*[-\u2500]+$", html)
  ]
  expect_error(
    find_session_info_in_html(url, html),
    "Cannot parse session info"
  )

  re_start <- "[-=\u2500\u2550][ ]Session info[ ]"
  html <- html[!grepl(re_start, html)]
  expect_error(
    find_session_info_in_html(url, html),
    "Cannot find session info"
  )
})

test_that("parse_url", {
  expect_snapshot(parse_url(
    "https://github.com/r-lib/sessioninfo/issues/6"
  ))
  expect_snapshot(parse_url(
    "https://github.com/r-lib/sessioninfo/issues/6#issuecomment-937772467"
  ))
})

test_that("get_session_info_literal", {
  si <- session_info()
  expect_equal(
    get_session_info_literal(si),
    list(arg = si, si = si, text = format(si))
  )

  lines <- format(si)
  expect_equal(
    get_session_info_literal(lines),
    list(arg = lines, si = lines, text = lines)
  )

  col <- paste0("\033[31m", lines, "\033[39m")
  expect_equal(
    get_session_info_literal(col),
    list(arg = col, si = col, text = lines)
  )

  str <- paste0(lines, collapse = "\n")
  expect_equal(
    get_session_info_literal(str),
    list(arg = str, si = str, text = lines)
  )

  ktr <- paste0("#> ", lines)
  expect_equal(
    get_session_info_literal(ktr),
    list(arg = ktr, si = ktr, text = lines)
  )

  ktr2 <- paste0("#&gt; ", lines)
  expect_equal(
    get_session_info_literal(ktr2),
    list(arg = ktr2, si = ktr2, text = lines)
  )

  expect_error(
    get_session_info_literal(structure(1, class = "foo")),
    "Could not interpret"
  )
})

test_that("strsplitx", {
  expect_equal(strsplitx("", "\n"), list(""))
})

test_that("check_session_info", {
  expect_warning(
    check_session_info("foo"),
    "This does not look like"
  )
  expect_silent(check_session_info("- Session info"))
  expect_silent(check_session_info("= Session info"))
  expect_silent(check_session_info("\u2500 Session info"))
  expect_silent(check_session_info("\u2550 Session info"))
})

test_that("beginning", {
  expect_snapshot(beginning("foo\nbar\nfoobar\nnotthis"))
  expect_snapshot(beginning(c("foo", "bar", "foobar", "notthis")))
  expect_snapshot(beginning(strrep("123456789 ", 20)))
})

test_that("session_diff_text", {
  x <- c("", "  ", " date 2020-01-01", "foo", "bar", "   ", "")
  y <- c(" date 2010-01-01", "foo", "baz", "")
  expect_snapshot(print(session_diff_text(x, y)))

  # if matching packages fails, we still have meaningful output
  local_mocked_bindings(expand_diff_text = function(...) stop())
  expect_snapshot(print(session_diff_text(x, y)))
})

test_that("diff_drop_empty", {
  cases <- list(
    list(character(), character()),
    list("foo", "foo"),
    list(c("", "foo"), "foo"),
    list(c("  ", "", "foo"), "foo"),
    list(c("", "foo", ""), "foo"),
    list(c("  ", "", "foo", "", "  "), "foo"),
    list(c("foo", "", "   "), "foo")
  )

  for (c in cases) {
    expect_equal(diff_drop_empty(c[[1]]), c[[2]], info = c[[1]])
  }
})

test_that("diff_no_date", {
  x <- c("foo", "date 2000-01-01", "date 2000-01-01")
  expect_equal(diff_no_date(x), x[-2])
  x2 <- c("foo", "   date 2000-01-01", "date 2000-01-01")
  expect_equal(diff_no_date(x2), x2[-2])
  x3 <- c("foo", "bar")
  expect_equal(diff_no_date(x3), x3)
})

test_that("diff_min_line", {
  x <- c("= 3456 ============", "- 345678 -----------", strrep("x", 1000))
  x2 <- gsub("=", "\u2550", gsub("-", "\u2500", x))
  expect_equal(diff_min_line(x), 19)
  expect_equal(diff_min_line(x2), 19)
  expect_equal(diff_min_line(strrep("-", 100)), 80)
})

test_that("diff_fix_lines", {
  x <- c("= 3456 ============", "foo", "- 345678 ----------", "bar")
  x2 <- gsub("=", "\u2550", gsub("-", "\u2500", x))
  exp <- c("= 3456 ===", "foo", "- 345678 -", "bar")
  exp2 <- gsub("=", "\u2550", gsub("-", "\u2500", exp))
  expect_equal(diff_fix_lines(x, 10), exp)
  expect_equal(diff_fix_lines(x2, 10), exp)

  withr::local_options(cli.unicode = TRUE)
  expect_equal(diff_fix_lines(x, 10), exp2)
  expect_equal(diff_fix_lines(x2, 10), exp2)
})

test_that("expand_diff_text", {
  lines1 <- readLines(test_path("fixtures", "lines1.txt"))
  lines2 <- readLines(test_path("fixtures", "lines2.txt"))
  xp1 <- expand_diff_text(lines1, lines2)
  expect_snapshot(xp1)

  xp2 <- expand_diff_text(lines2, lines1)
  expect_equal(xp1$old, xp2$new)
  expect_equal(xp1$new, xp2$old)

  expect_equal(
    expand_diff_text(lines1, "foobar"),
    list(old = lines1, new = "foobar")
  )
})

test_that("insert_instead", {
  cases <- list(
    list(1:10, 1,  1,  11:15, c(11:15, 2:10)),
    list(1:10, 1,  3,  11:15, c(11:15, 4:10)),
    list(1:10, 2,  2,  11:15, c(1, 11:15, 3:10)),
    list(1:10, 2,  5,  11:15, c(1, 11:15, 6:10)),
    list(1:10, 5,  10, 11:15, c(1:4, 11:15)),
    list(1:10, 10, 10, 11:15, c(1:9, 11:15)),
    list(1:10, 1,  10, 11:15, c(11:15)),
    list(1,    1,  1,  11:15, c(11:15)),
    list(1:10, 0,  0,  11:15, c(11:15, 1:10)),
    list(1:10, 11, 11, 11:15, c(1:10, 11:15)),
    list(1:10, 2,  1,  11:15, c(1, 11:15, 2:10))
  )

  for (i in seq_along(cases)) {
    c <- cases[[i]]
    expect_equal(
      insert_instead(c[[1]], c[[2]], c[[3]], c[[4]]),
      c[[5]],
      info = i
    )
  }
})

test_that("parse_pkgs", {
  lines <- readLines(test_path("fixtures", "lines4.txt"))
  pkgs <- parse_pkgs(lines)
  expect_equal(pkgs$begin, 17)
  expect_equal(pkgs$end, 40)
  expect_snapshot(names(pkgs$pkgs))
  expect_snapshot(pkgs$pkgs[["!"]])
  expect_snapshot(pkgs$pkgs$package)
  expect_snapshot(pkgs$pkgs)

  pkgs2 <- parse_pkgs(lines[1:40])
  expect_equal(pkgs, pkgs2)

  expect_null(parse_pkgs("foobar"))
})

test_that("parse_pkgs_section", {
  lines <- readLines(test_path("fixtures", "lines3.txt"))
  pkgs <- parse_pkgs_section(lines)
  expect_snapshot(names(pkgs))
  expect_snapshot(pkgs[["!"]])
  expect_snapshot(pkgs$package)
  expect_snapshot(pkgs)

  lines2 <- c(
    " package     * version    date (UTC) lib source",
    " cli           3.0.1.9000 2021-10-11 [1] local",
    " crayon        1.4.1      2021-02-08 [1] CRAN (R 4.1.0)",
    " prettycode    1.1.0      2019-12-16 [1] CRAN (R 4.1.0)",
    " prompt        1.0.0      2021-03-02 [1] local",
    " ps            1.6.0      2021-02-28 [1] CRAN (R 4.1.0)",
    " sessioninfo   1.1.1.9000 2021-10-12 [1] local",
    " withr         2.4.2      2021-04-18 [1] CRAN (R 4.1.0)"
  )
  pkgs2 <- parse_pkgs_section(lines2)
  expect_snapshot(names(pkgs2))
  expect_snapshot(pkgs2[["!"]])
  expect_snapshot(pkgs2$package)
  expect_snapshot(pkgs2)
})

test_that("find_word_lengths", {
  cases <- list(
    list("", numeric()),
    list(" ", numeric()),
    list("   ", numeric()),
    list("x", 2),
    list(" x", 3),
    list(" x ", 4),
    list("foo bar", c(4, 4)),
    list("foo    bar", c(7, 4)),
    list(" foo", 5),
    list("  foo   bar", c(8, 4))
  )
  for (c in cases) {
    expect_equal(find_word_lengths(c[[1]]), c[[2]], info = c[[1]])
  }
})

test_that("get_symbol_name", {
  expect_equal(get_symbol_name(as.symbol("x")), "x")
  expect_equal(get_symbol_name("x"), "x")
  expect_null(get_symbol_name(call("foo", "bar")))
})

#' @importFrom cli symbol

rule <- function(..., pad = NULL, double = FALSE) {
  if (is.null(pad)) pad <- if (double) symbol$double_line else symbol$line
  title <- if (length(list(...))) paste0(" ", ..., " ") else ""

  width <- max(cli::console_width() - cli::ansi_nchar(title, "width") - 3, 0)
  rule <- paste(pad, title, paste(rep(pad, width), collapse = ""), sep = "")
  cli::style_bold(cli::col_cyan(rule))
}

dash <- function(n = 2) {
  paste(rep(symbol$line, n), collapse = "")
}

cat_ln <- function(..., sep = "") {
  cat(..., "\n", sep = sep)
}

col_align <- function(x, align = c("left", "center", "right")) {
  x <- encodeString(x)
  mw <- max(cli::ansi_nchar(x, "width"))
  cli::ansi_align(x, align = align, width = mw)
}

format_column <- function(name, x) {
  col_align(c(name, x))
}

format_df <- function(x, highlighters = NULL) {
  cols <- mapply(
    names(x),
    x,
    FUN = format_column,
    USE.NAMES = FALSE,
    SIMPLIFY = FALSE
  )

  cols <- lapply(cols, function(x) {
    if (length(x) > 0) x[1] <- cli::col_grey(cli::style_italic(x[1]))
    x
  })

  for (idx in seq_along(highlighters)) {
    colname <- names(highlighters)[idx]
    colnum <- match(colname, names(x))
    if (is.na(colnum)) next
    cols[[colnum]][-1] <- highlighters[[idx]](cols[[colnum]][-1])
  }

  # remove trailing space, to avoid superfluous wrapping
  cli::ansi_trimws(do.call("paste", c("", cols)), "right")
}

highlight_version <- function(x) {
  ver <- tryCatch(package_version(trimws(x)), error = function(err) NULL)
  if (is.null(ver)) return(x)
  large <- vapply(ver, function(x) any(unlist(x) >= 1234), logical(1))
  x[large] <- cli::style_bold(cli::col_magenta(x[large]))
  x
}

#' Human readable name of the current operating system
#'
#' For example Windows 8.1 instead of Windows version 6.3.9600.
#' On macOS it includes the code names, on Linux it includes the
#' distribution names and codenames if appropriate.
#'
#' It uses [utils::sessionInfo()], but simplifies its output a bit
#' on Windows, to make it more concise.
#'
#' @return A character scalar.
#'
#' @export

os_name <- function() {
  x <- suppressWarnings(utils::sessionInfo("base")$running)
  if (is.null(x)) return(NA_character_)

  x <- gsub("Service Pack", "SP", x)

  x
}

#' Information about the currently loaded packages, or about a chosen set
#'
#' @param pkgs Which packages to show. It may be:
#'   * `NULL` or `"loaded"`: show all loaded packages,
#'   * `"attached"`: show all attached packages,
#'   * `"installed"`: show all installed packages,
#'   * a character vector of package names. Their (hard) dependencies are
#'     also shown by default, see the `dependencies` argument.
#' @param include_base Include base packages in summary? By default this is
#'   false since base packages should always match the R version.
#' @param dependencies Whether to include the (recursive) dependencies
#'   as well. See the `dependencies` argument of [utils::install.packages()].
#' @return A data frame with columns:
#'   * `package`: package name.
#'   * `ondiskversion`: package version (on the disk, which is sometimes
#'     not the same as the loaded version).
#'   * `loadedversion`: package version. This is the version of the loaded
#'     namespace if `pkgs` is `NULL`, and it is the version of the package
#'     on disk otherwise. The two of them are almost always the same,
#'     though.
#'   * `path`: path to the package on disk.
#'   * `loadedpath`: the path the package was originally loaded from.
#'   * `attached`: logical, whether the package is attached to the search
#'     path.
#'   * `is_base`: logical, whether the package is a base package.
#'   * `date`: the date the package was installed or built, in UTC.
#'   * `source`: where the package was installed from. E.g.
#'     `CRAN`, `GitHub`, `local` (from the local machine), etc.
#'   * `md5ok`: Whether MD5 hashes for package DLL files match, on Windows.
#'     `NA` on other platforms.
#'   * `library`: factor, which package library the package was loaded from.
#'     For loaded packages, this is (the factor representation of)
#'     `loadedpath`, for others `path`.
#'
#' See [session_info()] for the description of the *printed* columns
#' by `package_info` (as opposed to the *returned* columns).
#'
#' @export
#' @examplesIf FALSE
#' package_info()
#' package_info("sessioninfo")

package_info <- function(
  pkgs = c("loaded", "attached", "installed")[1],
  include_base = FALSE,
  dependencies = NA
) {
  if (is.null(pkgs)) pkgs <- "loaded"
  if (identical(pkgs, "!loaded") || identical(pkgs, "loaded")) {
    pkgs <- loaded_packages()
  } else if (identical(pkgs, "!attached") || identical(pkgs, "attached")) {
    pkgs <- attached_packages()
  } else if (identical(pkgs, "!installed") || identical(pkgs, "installed")) {
    pkgs <- installed_packages()
  } else {
    pkgs <- dependent_packages(pkgs, dependencies)
  }

  desc <- lapply(pkgs$package, pkg_desc, lib.loc = .libPaths())

  pkgs$is_base <- vapply(
    desc,
    function(x) identical(x$Priority, "base"),
    logical(1)
  )

  pkgs$date <- vapply(desc, pkg_date, character(1))
  pkgs$source <- vapply(desc, pkg_source, character(1))
  pkgs$md5ok <- vapply(desc, pkg_md5ok_dlls, logical(1))

  libpath <- pkg_lib_paths()
  path <- ifelse(is.na(pkgs$loadedpath), pkgs$path, pkgs$loadedpath)
  pkgs$library <- factor(dirname(path), levels = libpath)

  if (!include_base) pkgs <- pkgs[!pkgs$is_base, ]

  rownames(pkgs) <- pkgs$package
  class(pkgs) <- c("packages_info", "data.frame")
  pkgs
}

pkg_desc <- function(package, lib.loc = NULL) {
  desc <- suppressWarnings(
    utils::packageDescription(package, lib.loc = lib.loc)
  )
  if (inherits(desc, "packageDescription")) desc else NULL
}

pkg_lib_paths <- function() {
  normalizePath(.libPaths(), winslash = "/")
}

pkg_date <- function(desc) {
  if (!is.null(desc$`Date/Publication`)) {
    date <- desc$`Date/Publication`
  } else if (!is.null(desc$Built)) {
    built <- strsplit(desc$Built, "; ")[[1]]
    date <- built[3]
  } else {
    date <- NA_character_
  }

  as.character(as.Date(strptime(date, "%Y-%m-%d")))
}

pkg_source <- function(desc) {
  if (is.null(desc)) {
    NA_character_
  } else if (!is.null(desc$GithubSHA1)) {
    str <- paste0(
      "Github (",
      desc$GithubUsername,
      "/",
      desc$GithubRepo,
      "@",
      desc$GithubSHA1,
      ")"
    )
  } else if (!is.null(desc$RemoteType) && desc$RemoteType == "standard") {
    if (!is.null(desc$Repository) && desc$Repository == "CRAN") {
      pkg_source_cran(desc)
    } else if (!is.null(desc$Repository)) {
      str_trim(desc$Repository, 10)
    } else if (!is.null(desc$biocViews) && desc$biocViews != "") {
      "Bioconductor"
    } else {
      "Custom"
    }
  } else if (!is.null(desc$RemoteType) && desc$RemoteType != "cran") {
    # want to generate these:
    # remoteType (username/repo@commit)
    # remoteType (username/repo)
    # remoteType (@commit)
    # remoteType
    remote_type <- desc$RemoteType

    # RemoteUsername and RemoteRepo should always be present together
    if (!is.null(desc$RemoteUsername) && (!is.null(desc$RemoteRepo))) {
      user_repo <- paste0(desc$RemoteUsername, "/", desc$RemoteRepo)
    } else if (!is.null(desc$RemoteUrl)) {
      user_repo <- desc$RemoteUrl
    } else {
      user_repo <- NULL
    }

    if (!is.null(desc$RemoteSha)) {
      sha <- paste0("@", desc$RemoteSha)
    } else {
      sha <- NULL
    }

    # in order to fulfill the expectation of formatting, we paste the user_repo
    # and sha together
    if (!is.null(user_repo) || !is.null(sha)) {
      user_repo_and_sha <- paste0(" (", user_repo, sha, ")")
    } else {
      user_repo_and_sha <- NULL
    }

    str <- paste0(remote_type, user_repo_and_sha)
  } else if (!is.null(desc$Repository)) {
    pkg_source_cran(desc)
  } else if (!is.null(desc$biocViews) && desc$biocViews != "") {
    "Bioconductor"
  } else if (
    isNamespaceLoaded(desc$Package) &&
      !is.null(asNamespace(desc$Package)$.__DEVTOOLS__)
  ) {
    "load_all()"
  } else {
    "local"
  }
}

pkg_source_cran <- function(desc) {
  repo <- desc$Repository

  if (!is.null(desc$Built)) {
    built <- strsplit(desc$Built, "; ")[[1]]
    ver <- sub("$R ", "", built[1])
    repo <- paste0(repo, " (", ver, ")")
  }

  repo
}

pkg_md5ok_dlls <- function(desc) {
  if (is.null(desc)) return(NA)
  if (.Platform$OS.type != "windows") return(NA)
  pkgdir <- dirname(dirname(attr(desc, "file")))
  if (!file.exists(file.path(pkgdir, "libs"))) return(TRUE)
  stored <- pkg_md5_stored(pkgdir)
  if (is.null(stored)) return(NA)
  disk <- pkg_md5_disk(pkgdir)
  identical(stored, disk)
}

pkg_md5_stored <- function(pkgdir) {
  md5file <- file.path(pkgdir, "MD5")
  md5 <- tryCatch(
    suppressWarnings(readLines(md5file)),
    error = function(e) NULL
  )
  if (is.null(md5)) return(NULL)
  hash <- sub(" .*$", "", md5)
  filename <- sub("^[^ ]* \\*", "", md5)
  dll <- grep("[dD][lL][lL]$", filename)
  order_by_name(structure(hash[dll], names = tolower(filename[dll])))
}

pkg_md5_disk <- function(pkgdir) {
  old <- getwd()
  on.exit(setwd(old), add = TRUE)
  setwd(pkgdir)
  dll_files <- file.path(
    "libs",
    dir("libs", pattern = "[dD][lL][lL]$", recursive = TRUE)
  )
  md5_files <- tools::md5sum(dll_files)
  order_by_name(structure(unname(md5_files), names = tolower(dll_files)))
}

abbrev_long_sha <- function(x) {
  sub("([0-9a-f]{7})[0-9a-f]{33}", "\\1", x)
}

#' @export

format.packages_info <- function(x, ...) {
  if (nrow(x) == 0) {
    return(cli::col_grey("No packages."))
  }

  unloaded <- is.na(x$loadedversion)
  flib <- function(x) ifelse(is.na(x), "?", as.integer(x))

  px <- data.frame(
    package = x$package,
    "*" = ifelse(x$attached, "*", ""),
    version = ifelse(unloaded, x$ondiskversion, x$loadedversion),
    "date (UTC)" = x$date,
    lib = paste0("[", flib(x$library), "]"),
    source = abbrev_long_sha(x$source),
    stringsAsFactors = FALSE,
    check.names = FALSE
  )

  anyattached <- any(x$attached)
  badloaded <- package_version(x$loadedversion, strict = FALSE) !=
    package_version(x$ondiskversion, strict = FALSE)
  badloaded <- !is.na(badloaded) & badloaded

  px$source <- ifelse(
    badloaded,
    paste0(px$source, " (on disk ", x$ondiskversion, ")"),
    px$source
  )

  badmd5 <- !is.na(x$md5ok) & !x$md5ok

  badpath <- !is.na(x$loadedpath) & x$loadedpath != x$path

  baddel <- is.na(x$ondiskversion)
  badpath[baddel] <- FALSE

  if (any(badloaded) || any(badmd5) || any(badpath) || any(baddel)) {
    prob <- paste0(
      ifelse(badloaded, "V", ""),
      ifelse(badpath, "P", ""),
      ifelse(badmd5, "D", ""),
      ifelse(baddel, "R", "")
    )
    px <- cbind("!" = prob, px)
  }

  dng <- function(x) cli::bg_red(cli::col_white(x))

  highlighters <- list(
    "!" = function(x) {
      ifelse(empty(x), x, dng(x))
    },
    version = function(x) {
      highlight_version(x)
    },
    "date (UTC)" = function(x) {
      cli::col_grey(x)
    },
    lib = function(x) {
      cli::col_grey(x)
    },
    source = function(x) {
      common <- grepl("^(Bioconductor|CRAN)", x)
      x[!common] <- cli::style_bold(cli::col_magenta(x[!common]))
      x[common] <- cli::col_grey(x[common])
      x
    }
  )

  fmt <- c(format_df(px, highlighters = highlighters), "")

  lapply(
    seq_along(levels(x$library)),
    function(i) {
      fmt <<- c(fmt, cli::col_grey(paste0(" [", i, "] ", levels(x$library)[i])))
    }
  )

  if ("!" %in% names(px)) fmt <- c(fmt, "")
  if (anyattached) {
    fmt <- c(
      fmt,
      paste0(
        " ",
        dng("*"),
        " ",
        dash(2),
        " Packages attached to the search path."
      )
    )
  }
  if (any(badloaded)) {
    fmt <- c(
      fmt,
      paste0(
        " ",
        dng("V"),
        " ",
        dash(2),
        " Loaded and on-disk version mismatch."
      )
    )
  }
  if (any(badpath)) {
    fmt <- c(
      fmt,
      paste0(" ", dng("P"), " ", dash(2), " Loaded and on-disk path mismatch.")
    )
  }
  if (any(badmd5)) {
    fmt <- c(
      fmt,
      paste0(
        " ",
        dng("D"),
        " ",
        dash(2),
        " DLL MD5 mismatch, broken installation."
      )
    )
  }
  if (any(baddel)) {
    fmt <- c(
      fmt,
      paste0(" ", dng("R"), " ", dash(2), " Package was removed from disk.")
    )
  }

  fmt
}

#' @export

print.packages_info <- function(x, ...) {
  cat(format(x, ...), sep = "\n")
}

#' @export

as.character.packages_info <- function(x, ...) {
  old <- options(cli.num_colors = 1)
  on.exit(options(old), add = TRUE)
  format(x, ...)
}

test_that("package_info, loaded", {

  descs <- readRDS("fixtures/devtools-deps.rda")
  alldsc <- readRDS("fixtures/descs.rda")
  exp <- readRDS(paste0("fixtures/devtools-info-", .Platform$OS.type, ".rda"))

  local_mocked_bindings(loaded_packages = function() descs)
  local_mocked_bindings(pkg_desc = function(x, ...) alldsc[[x]])
  local_mocked_bindings(pkg_lib_paths = function() levels(exp$library))

  pi <- package_info()
  expect_identical(pi, exp)
})

test_that("package_info, dependent", {

  descs <- readRDS("fixtures/devtools-deps.rda")
  alldsc <- readRDS("fixtures/descs.rda")
  exp <- readRDS(paste0("fixtures/devtools-info-", .Platform$OS.type, ".rda"))

  local_mocked_bindings(dependent_packages = function(...) descs)
  local_mocked_bindings(pkg_desc = function(x, ...) alldsc[[x]])
  local_mocked_bindings(pkg_lib_paths = function() levels(exp$library))

  pi <- package_info("devtools")
  expect_identical(pi, exp)
})

test_that("pkg_date", {
  crayon <- readRDS("fixtures/descs.rda")$crayon
  expect_equal(pkg_date(crayon), "2016-06-28")

  crayon$`Date/Publication` <- NULL
  expect_equal(pkg_date(crayon), "2016-06-29")

  crayon$`Built` <- NULL
  expect_identical(pkg_date(crayon), NA_character_)
})

test_that("pkg_source", {
  descs <- readRDS("fixtures/descs.rda")

  expect_identical(
    pkg_source(descs$debugme),
    "Github (gaborcsardi/debugme@df8295a75f8e26bec90386beb1c82b529427029f)"
  )

  expect_identical(pkg_source(descs$curl), "CRAN (R 3.3.3)")

  biobase <- readRDS("fixtures/biobase.rda")
  expect_identical(pkg_source(biobase), "Bioconductor")

  memoise <- readRDS("fixtures/memoise.rda")
  expect_identical(pkg_source(memoise), "CRAN (R 3.3.3)")
})

test_that("pkg_source edge case, remote repo, but no RemoteSha", {
  desc <- readRDS("fixtures/no-sha.rda")
  expect_identical(pkg_source(desc), "github (r-pkgs/desc)")
})

test_that("pkg_source edge case, remote repo, no RemoteRepo", {
  desc <- readRDS("fixtures/no-remote-repo.rda")
  expect_identical(pkg_source(desc), "github")
})

test_that("pkg_md5_stored", {
  md5 <- pkg_md5_stored("fixtures")
  exp <- c(`libs/i386/fansi.dll` = "7b96ab4bf019b0cfed86425634d640e8",
           `libs/x64/fansi.dll` = "6503170d698e5a7916bf2457edc5de8d")
  expect_identical(md5, exp)
})

test_that("pkg_md5_disk", {
  dir.create(tmp <- tempfile())
  on.exit(unlink(tmp, recursive = TRUE), add = TRUE)
  dir.create(file.path(tmp, "libs"))
  dir.create(file.path(tmp, "libs", "i386"))
  dir.create(file.path(tmp, "libs", "x64"))
  writeBin(charToRaw("foo\n"), con = file.path(tmp, "libs", "i386", "foo.dll"))
  writeBin(charToRaw("bar\n"), con = file.path(tmp, "libs", "x64", "foo.dll"))
  md5 <- pkg_md5_disk(tmp)

  exp <- c(`libs/i386/foo.dll` = "d3b07384d113edec49eaa6238ad5ff00",
           `libs/x64/foo.dll` = "c157a79031e1c40f85931829bc5fc552")
  expect_identical(md5,exp)
})

test_that("print.packages_info", {
  info <- readRDS(paste0("fixtures/devtools-info-", .Platform$OS.type, ".rda"))
  expect_output(
    print(info), "package    * version     date (UTC) lib source",
    fixed = TRUE
  )
})

test_that("print.packages_info ignores max.print", {
  info <- readRDS(paste0("fixtures/devtools-info-", .Platform$OS.type, ".rda"))
  withr::local_options(list(max.print = 1))
  out <- capture_output(print(info))
  out <- tail(strsplit(out, split = "\r?\n")[[1]], -1)
  expect_length(out, nrow(info) + 3)
})

# taken from usethis
# definitely designed for GitHub URLs but not overtly GitHub-specific
# https://stackoverflow.com/questions/2514859/regular-expression-for-git-repository
# https://git-scm.com/docs/git-clone#_git_urls
# https://stackoverflow.com/questions/27745/getting-parts-of-a-url-regex
github_url_regex <- paste0(
  "^",
  "(?<protocol>\\w+://)?",
  "(?<user>.+@)?",
  "(?<host>[^/:]+)",
  "[/:]",
  "(?<repo_owner>[^/]+)",
  "/",
  "(?<repo_name>[^/#]+)",
  "(?<fragment>.*)",
  "$"
)

github_fragment_regex <- paste0(
  "/runs/",
  "(?<run_id>[0-9]+)",
  "/jobs?/",
  "(?<html_id>[0-9]+)",
  "$"
)

parse_as_gha_url <- function(url) {
  res <- re_match(url, github_url_regex)$groups
  res2 <- re_match(res$fragment, github_fragment_regex)$groups
  res$run_id <- res2$run_id
  res$html_id <- res2$html_id

  ok <- res$host %in% "github.com" &
    !is.na(res$repo_owner) & !is.na(res$repo_name) &
    !is.na(res$run_id) & !is.na(res$html_id)

  data.frame(
    owner  = ifelse(ok, res$repo_owner, NA_character_),
    repo   = ifelse(ok, res$repo_name, NA_character_),
    run_id = ifelse(ok, res$run_id, NA_character_),
    html_id = ifelse(ok, res$html_id, NA_character_),
    stringsAsFactors = FALSE,
    row.names = NULL
  )
}

is_gha_url <- function(url) {
  res <- parse_as_gha_url(url)
  !is.na(res$run_id)
}

get_session_info_gha <- function(url) {
  if (!requireNamespace("gh", quietly = TRUE)) {
    stop(
      "The gh package is not available.\n",
      "This appears to be the URL for a GitHub Actions (GHA) log:\n",
      url, "\n",
      "You must install the gh package to get session info for GHA job logs."
    )
  }

  dat <- parse_as_gha_url(url)
  # the last ID in the browser URL is not the job_id!
  # instead, we must lookup the job_id
  jobs <- gh::gh(
    "/repos/{owner}/{repo}/actions/runs/{run_id}/jobs",
    owner = dat$owner, repo = dat$repo, run_id = dat$run_id
  )
  html_urls <- vapply(jobs[["jobs"]], function(x) x[["html_url"]], "")
  i <- which(html_urls == url)
  if (length(i) == 0) {
    stop("Failed to find the job associated with '", url, "'; perhaps there was a later attempt?")
  }
  dat$job_id <- jobs[["jobs"]][[i]][["id"]]

  meta <- gh::gh(
    "/repos/{owner}/{repo}/actions/jobs/{job_id}",
    owner = dat$owner, repo = dat$repo, job_id = dat$job_id
  )
  raw_log <- gh::gh(
    "/repos/{owner}/{repo}/actions/jobs/{job_id}/logs",
    owner = dat$owner, repo = dat$repo, job_id = dat$job_id
  )
  timestamped_lines <- unlist(strsplit(raw_log$message, split = "\r\n|\n"))
  lines <- sub("^[^\\s]+\\s", "", timestamped_lines, perl = TRUE)

  re_start <- "[-=\u2500\u2550][ ]Session info[ ]"
  cand <- grep(re_start, lines)
  if (length(cand) == 0) stop("Cannot find session info at '", url, "'.")
  lines <- lines[cand[1]:length(lines)]
  lines[1] <- sub(paste0("^.*(", re_start, ")"), "\\1", lines[1])

  grepl_end <- function(lines) {
    grepl("^[ ]*\\[[0-9]\\] ", lines) |
      grepl("^[ ]*[-\u2500]+$", lines)
  }
  end <- which(grepl_end(lines))[1]
  if (is.na(end)) stop("Cannot parse session info from '", url, "'.")
  while (end < length(lines) && grepl_end(lines[end + 1])) {
    end <- end + 1
  }

  si <- get_session_info_literal(lines[1:end])

  si$arg <- "github-actions"
  si$name <- paste(
    meta$name,
    meta$conclusion,
    paste("job", meta$id),
    paste("run", meta$run_id),
    sep = " | "
  )
  si
}

get_os <- function() {
  if (.Platform$OS.type == "windows") {
    "win"
  } else if (Sys.info()["sysname"] == "Darwin") {
    "mac"
  } else {
    "unix"
  }
}

clipboard_read <- function() {
  os <- get_os()

  switch(
    os,
    win = utils::readClipboard(),
    mac = clipboard_read_mac(),
    clipboard_read_x11()
  )
}

clipboard_read_mac <- function() {
  on.exit(try(close(con), silent = TRUE), add = TRUE)
  con <- pipe("pbpaste")
  scan(con, what = "", sep = "\n", blank.lines.skip = FALSE, quiet = TRUE)
}

clipboard_read_x11 <- function() {
  on.exit(try(close(con), silent = TRUE), add = TRUE)
  con <- file("clipboard")
  scan(con, what = "", sep = "\n", blank.lines.skip = FALSE, quiet = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package-info.R
\name{package_info}
\alias{package_info}
\title{Information about the currently loaded packages, or about a chosen set}
\usage{
package_info(
  pkgs = c("loaded", "attached", "installed")[1],
  include_base = FALSE,
  dependencies = NA
)
}
\arguments{
\item{pkgs}{Which packages to show. It may be:
\itemize{
\item \code{NULL} or \code{"loaded"}: show all loaded packages,
\item \code{"attached"}: show all attached packages,
\item \code{"installed"}: show all installed packages,
\item a character vector of package names. Their (hard) dependencies are
also shown by default, see the \code{dependencies} argument.
}}

\item{include_base}{Include base packages in summary? By default this is
false since base packages should always match the R version.}

\item{dependencies}{Whether to include the (recursive) dependencies
as well. See the \code{dependencies} argument of \code{\link[utils:install.packages]{utils::install.packages()}}.}
}
\value{
A data frame with columns:
\itemize{
\item \code{package}: package name.
\item \code{ondiskversion}: package version (on the disk, which is sometimes
not the same as the loaded version).
\item \code{loadedversion}: package version. This is the version of the loaded
namespace if \code{pkgs} is \code{NULL}, and it is the version of the package
on disk otherwise. The two of them are almost always the same,
though.
\item \code{path}: path to the package on disk.
\item \code{loadedpath}: the path the package was originally loaded from.
\item \code{attached}: logical, whether the package is attached to the search
path.
\item \code{is_base}: logical, whether the package is a base package.
\item \code{date}: the date the package was installed or built, in UTC.
\item \code{source}: where the package was installed from. E.g.
\code{CRAN}, \code{GitHub}, \code{local} (from the local machine), etc.
\item \code{md5ok}: Whether MD5 hashes for package DLL files match, on Windows.
\code{NA} on other platforms.
\item \code{library}: factor, which package library the package was loaded from.
For loaded packages, this is (the factor representation of)
\code{loadedpath}, for others \code{path}.
}

See \code{\link[=session_info]{session_info()}} for the description of the \emph{printed} columns
by \code{package_info} (as opposed to the \emph{returned} columns).
}
\description{
Information about the currently loaded packages, or about a chosen set
}
\examples{
\dontshow{if (FALSE) withAutoprint(\{ # examplesIf}
package_info()
package_info("sessioninfo")
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sessioninfo-package.R
\docType{package}
\name{sessioninfo-package}
\alias{sessioninfo}
\alias{sessioninfo-package}
\title{sessioninfo: R Session Information}
\description{
Query and print information about the current R session. It is similar to 'utils::sessionInfo()', but includes more information about packages, and where they were installed from.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://sessioninfo.r-lib.org}
  \item \url{https://github.com/r-lib/sessioninfo}
  \item Report bugs at \url{https://github.com/r-lib/sessioninfo/issues}
}

}
\author{
\strong{Maintainer}: Gábor Csárdi \email{csardi.gabor@gmail.com}

Authors:
\itemize{
  \item Hadley Wickham
  \item Winston Chang
  \item Robert Flight
  \item Kirill Müller
  \item Jim Hester
}

Other contributors:
\itemize{
  \item R Core team [contributor]
  \item Posit Software, PBC (\href{https://ror.org/03wc8by49}{ROR}) [copyright holder, funder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/session-info.R
\name{session_info}
\alias{session_info}
\title{Print session information}
\usage{
session_info(
  pkgs = c("loaded", "attached", "installed")[1],
  include_base = FALSE,
  info = c("auto", "all", "platform", "packages", "python", "external"),
  dependencies = NA,
  to_file = FALSE
)
}
\arguments{
\item{pkgs}{Which packages to show. It may be:
\itemize{
\item \code{NULL} or \code{"loaded"}: show all loaded packages,
\item \code{"attached"}: show all attached packages,
\item \code{"installed"}: show all installed packages,
\item a character vector of package names. Their (hard) dependencies are
also shown by default, see the \code{dependencies} argument.
}}

\item{include_base}{Include base packages in summary? By default this is
false since base packages should always match the R version.}

\item{info}{What information to show, it can be \code{"auto"} to choose
automatically, \code{"all"} to show everything, or a character vector
with elements from:
\itemize{
\item \code{"platform"}: show platform information via \code{\link[=platform_info]{platform_info()}},
\item \code{"packages"}: show package information via \code{\link[=package_info]{package_info()}},
\item \code{"python"}: show Python configuration via \code{\link[=python_info]{python_info()}},
\item \code{"external"}: show information about external software, via
\code{\link[=external_info]{external_info()}}.
}}

\item{dependencies}{Whether to include the (recursive) dependencies
as well. See the \code{dependencies} argument of \code{\link[utils:install.packages]{utils::install.packages()}}.}

\item{to_file}{Whether to print the session information to a file.
If \code{TRUE} the name of the file will be \code{session-info.txt}, but
\code{to_file} may also be a string to specify the file name.}
}
\value{
A \code{session_info} object.

If \code{to_file} is not \code{FALSE} then it is
returned invisibly. (To print it to both a file and to the screen,
use \code{(session_info(to_file = TRUE))}.)
}
\description{
This is \code{\link[utils:sessionInfo]{utils::sessionInfo()}} re-written from scratch to both exclude
data that's rarely useful (e.g., the full collate string or base packages
loaded) and include stuff you'd like to know (e.g., where a package was
installed from).
}
\details{
Columns in the \emph{printed} package list:
\itemize{
\item \code{package}: package name
\item \code{*}: whether the package is attached to the search path
\item \code{version}: package version. If the version is marked with \verb{(!)} that
means that the loaded and the on-disk version of the package are
different.
\item \code{date}: when the package was built, if this information is available.
This is the \code{Date/Publication} or the \code{Built} field from
\code{DESCRIPTION}. (These are usually added automatically by R.)
Sometimes this data is not available, then it is \code{NA}.
\item \code{source}: where the package was built or installed from, if available.
Examples: \verb{CRAN (R 3.3.2)}, \verb{Github (r-lib/pkgbuild@8aab60b)},
\code{Bioconductor}, \code{local}.
}

See \code{\link[=package_info]{package_info()}} for the list of columns in the data frame that
is \emph{returned} (as opposed to \emph{printed}).
}
\examples{
\dontshow{if (FALSE) withAutoprint(\{ # examplesIf}
session_info()
session_info("sessioninfo")
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compare.R
\name{session_diff}
\alias{session_diff}
\title{Compare session information from two sources}
\usage{
session_diff(
  old = "local",
  new = "clipboard",
  packages = c("diff", "merge"),
  ...
)
}
\arguments{
\item{old, new}{A \code{session_info} object (the return value of
\code{\link[=session_info]{session_info()}}), or a pointer to \code{\link[=session_info]{session_info()}} output. See details
below.}

\item{packages}{How to compare the package info for \code{old} and \code{new}:
\itemize{
\item \code{"diff"}: line diffs, in the style of \code{diff -u}
\item \code{"merge"}: merge the information into one data frame, with one row per
package. Only package \code{version} and \code{source} are compared.
}}

\item{...}{Passed to any new \code{\link[=session_info]{session_info()}} calls.}
}
\description{
Compare session information from two sources
}
\details{
Various way to specify \code{old} and \code{new}:
\itemize{
\item A \code{session_info} object.
\item \code{"local"} runs \code{\link[=session_info]{session_info()}} in the current
session, and uses its output.
\item \code{"clipboard"} takes the session info from the system clipboard.
If the clipboard contains a URL, it is followed to download the
session info.
\item The URL where you inspect the results for a GitHub Actions job.
Typically has this form:

\if{html}{\out{<div class="sourceCode">}}\preformatted{https://github.com/OWNER/REPO/actions/runs/RUN_ID/jobs/HTML_ID
}\if{html}{\out{</div>}}

Internally, this URL is parsed so we can look up the job id, get the
log file, and extract session info.
\item Any other URL starting with \verb{http://} or \verb{https://}. \code{session_diff()}
searches the HTML (or text) page for the session info header to find the
session info.
}
}
\examples{
\dontshow{if (FALSE) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
session_diff()
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/external-info.R
\name{external_info}
\alias{external_info}
\title{Information about related software}
\usage{
external_info()
}
\value{
A list with elements:
\itemize{
\item \code{cairo}: The cairo version string.
\item \code{libpng}: The png version string.
\item \code{jpeg}: The jpeg version string.
\item \code{tiff}: The tiff library and version string used.
\item \code{tcl}: The tcl version string.
\item \code{curl}: The curl version string.
\item \code{zlib}: The zlib version string.
\item \code{bzlib}: The zlib version string.
\item \code{xz}: The zlib version string.
\item \code{PCRE}: The Perl Compatible Regular Expressions (PCRE) version string.
\item \code{ICU}: The International Components for Unicode (ICU) version string.
\item \code{TRE}: The TRE version string.
\item \code{iconv}: The iconv version string.
\item \code{readline}: The readline version string.
\item \code{BLAS}: The path with the implementation of BLAS in use.
\item \code{LAPACK}: The path with the implementation of LAPACK in use.
}
}
\description{
Information about related software
}
\details{
Note that calling this function will attempt to load the tcltk and
grDevices packages.
}
\examples{
\dontshow{if (FALSE) withAutoprint(\{ # examplesIf}
external_info()
\dontshow{\}) # examplesIf}
}
\seealso{
Similar functions and objects in the base packages:
\code{\link[utils:sessionInfo]{utils::sessionInfo()}}, \link[base:extSoftVersion]{base::extSoftVersion}, \code{\link[tcltk:tclVersion]{tcltk::tclVersion()}}
\link[base:La_library]{base::La_library}, \code{\link[base:La_version]{base::La_version()}}, \code{\link[base:libcurlVersion]{base::libcurlVersion()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/python-info.R
\name{python_info}
\alias{python_info}
\title{Python configuration}
\usage{
python_info()
}
\value{
Returns a \link[reticulate:py_config]{reticulate::py_config} object, which also has the
\code{python_info} class. It is a named list of values.

If reticulate is not installed or Python is not configured,
then it return a \code{python_info} object that is a character vector, and
it does not have a \code{py_config} class.
}
\description{
Python configuration
}
\examples{
\dontshow{if (FALSE) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
python_info()
session_info(info = "all")
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/platform-info.R
\name{platform_info}
\alias{platform_info}
\title{Information about the current platform}
\usage{
platform_info()
}
\value{
A list with elements:
\itemize{
\item \code{version}: the R version string.
\item \code{os}: the OS name in human readable format, see \code{\link[=os_name]{os_name()}}.
\item \code{system}: CPU, and machine readable OS name, separated by a comma.
\item \code{ui}: the user interface, e.g. \code{Rgui}, \code{RTerm}, etc. see \code{GUI}
in \link[base:Platform]{base::.Platform}.
\item \code{hostname}: the name of the machine known on the network, see
\code{nodename} in \code{\link[base:Sys.info]{base::Sys.info()}}. For privacy, it is only included
if the \code{sessioninfo.include_hostname} option is set to \code{TRUE}.
\item \code{language}: The current language setting. The \code{LANGUAGE} environment
variable, if set, or \code{(EN)} if unset.
\item \code{collate}: Collation rule, from the current locale.
\item \code{ctype}: Native character encoding, from the current locale.
\item \code{tz}: The current time zone.
\item \code{date}: The current date.
\item \code{rstudio}: RStudio format string, only added in RStudio.
\item \code{pandoc}: pandoc version and path
\item \code{quarto}: quarto version and path
}
}
\description{
Information about the current platform
}
\examples{
\dontshow{if (FALSE) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
platform_info()
\dontshow{\}) # examplesIf}
}
\seealso{
Similar functions and objects in the base packages:
\link[base:Version]{base::R.version.string}, \code{\link[utils:sessionInfo]{utils::sessionInfo()}}, \link[base:Version]{base::version},
\link[base:Platform]{base::.Platform}, \code{\link[base:locales]{base::Sys.getlocale()}}, \code{\link[base:timezones]{base::Sys.timezone()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/osname.R
\name{os_name}
\alias{os_name}
\title{Human readable name of the current operating system}
\usage{
os_name()
}
\value{
A character scalar.
}
\description{
For example Windows 8.1 instead of Windows version 6.3.9600.
On macOS it includes the code names, on Linux it includes the
distribution names and codenames if appropriate.
}
\details{
It uses \code{\link[utils:sessionInfo]{utils::sessionInfo()}}, but simplifies its output a bit
on Windows, to make it more concise.
}
